################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
mali_evaluate.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python mali_evaluate.py --help

Type::

   python mali_evaluate.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import math

USAGE="""python %s [OPTIONS] < exonerate_output > filtered

Evaluate a multiple alignment. Remove gaps if so wished.

Version = $Id: mali_evaluate.py 2782 2009-09-10 11:40:29Z andreas $

Options:
-h, --help                      print this message.
-v, --verbose=                  loglevel.
-o, --output-pattern            pattern for output multiple alignments
-p, --master-pattern=           pattern identifying sequences of the query species from identifier
-s, --species-pattern=          pattern identifying species from identifier
-e, --exons=                    exon information for evaluation
-c, --cluster                   cluster by overlap
-f, --remove-fragments          remove fragments
-p, --prefix=                   prefix for cluster identifier
--min-overlap-percent           minimum percent overlap between sequence paris
--min-overlap-residues          minimum overlap in residues between sequence pairs
--min-coverage-percent          minimum percent coverage between sequence paris
--min-coverage-residues         minimum coverage in residues between sequence pairs
--components                    filename with components to be analyses separately in the multiple alignment
""" % sys.argv[0]

param_long_options=["verbose=", "help", "file-output=",
                    "min-overlap-residues=", "min-overlap-percent=",
                    "min-coverage-residues=", "min-coverage-percent=",
                    "components=",
                    "species-pattern=", "master-pattern=", "output-pattern=",
                    "exons=", "cluster", "remove-fragments", "prefix=",
                    "version"
                    ]

param_short_options="v:ho:s:p:e:"

param_loglevel = 1

param_gap_char = "-"
param_mask_char = "x"

param_output_pattern = None

param_filename_exons = None
param_filename_components = None

param_min_overlap_residues = 10
param_min_overlap_percent  = 0.5

param_min_coverage_residues = 10
param_min_coverage_percent  = 0.5


param_max_exons_difference = 1

param_species_pattern = "^([^|:@]+)[^|:@]"
param_master_pattern = None

param_threshold_splipping_exon_boundary = 9
param_evaluate_single_exons_min_coverage = 80
param_evaluate_min_percent_exon_identity = 70

## clustering parameters
param_clustering_max_distance = 20
param_max_compatibility = 50

## permissiveness for merging clusters, set to < 0 for disallowing merging
param_cluster_merging_max_distance = 2

param_do_cluster = False

param_remove_fragments = False

param_pattern_prefix = ""

import CGAT.Experiment as E
import CGAT.MaliIO as MaliIO
import scipy
import CGAT.Exons as Exons
import alignlib
import CGAT.Genomics as Genomics
import numpy

##--------------------------------------------------------------------------------------- 
def GetOverlapMatrix( mali, identifiers, gap_char = "-" ):
    """get overlap matrix for all pairwise combinations in mali.
    """
    wmali = len(identifiers)
    lmali = len(mali[identifiers[0]])

    coverages = numpy.zeros( wmali )

    matrix_overlap = numpy.zeros( (wmali, wmali) )
    matrix_total = numpy.ones( (wmali, wmali) )    
    
    for x in range(wmali):
        a = mali[identifiers[x]]

        ncoverage = 0
        for i in range(lmali):
            if a[i] != param_gap_char:
                  ncoverage += 1
                
        coverages[x] = ncoverage 

        for y in range(x+1, wmali):
            
            noverlap = 0
            
            b = mali[identifiers[y]]
            ntotal = 0
            for i in range(lmali):
                if a[i] != gap_char and b[i] != gap_char and \
                       a[i] in string.uppercase and b[i] in string.uppercase:
                    noverlap += 1
                if a[i] != gap_char or b[i] != gap_char:
                    ntotal += 1
                    
            matrix_total[x][y] = ntotal
            matrix_total[y][x] = ntotal
            matrix_overlap[x][y] = noverlap
            matrix_overlap[y][x] = noverlap

    return matrix_overlap, matrix_total, coverages

##--------------------------------------------------------------------------------------- 
def WriteOverlap( mali, identifiers, prefix = "" ):

    """check multiple alignment for overlap.

    This subroutine dumps out various summary parameters.
    """

    if len(identifiers) <= 1: return []
    
    wmali = len(identifiers)
    lmali = len(mali[identifiers[0]])

    ## check for non-overlapping pairs
    nnon_overlapping = 0
    nfragments = 0
    pairs = []
    ppairs = []
    coverages = []
    fragments = []

    for x in range(wmali):
        a = mali[identifiers[x]]

        ncoverage = 0
        for i in range(lmali):
            if a[i] != param_gap_char:
                  ncoverage += 1
                
        coverages.append( ncoverage )

        if ncoverage < param_min_coverage_residues or (float(ncoverage) / lmali) < param_min_overlap_percent:
            if param_loglevel == 3:
                print "# fragment: %s\t%i\t%i\t%.2f" % (identifiers[x], ncoverage, lmali, float(ncoverage) / lmali )
            
            nfragments += 1
            fragments.append( identifiers[x] )
                
        for y in range(x+1, wmali):
            
            if x == y: continue
            
            noverlap = 0
            
            b = mali[identifiers[y]]
            ntotal = 0
            for i in range(lmali):
                if a[i] != param_gap_char and b[i] != param_gap_char:
                    noverlap += 1
                if a[i] != param_gap_char or b[i] != param_gap_char:
                    ntotal += 1
                    
            pairs.append( noverlap )
            ppairs.append( float(noverlap) / ntotal )
            if param_loglevel >= 5:
                print "# overlap: %s\t%s\t%i\t%i\t%.2f" % (identifiers[x], identifiers[y], noverlap, ntotal, float(noverlap) / ntotal )
            
            if noverlap < param_min_overlap_residues or (float(noverlap) / ntotal) < param_min_overlap_percent:
                nnon_overlapping += 1
            
                if param_loglevel == 3:
                    print "# failed overlap: %s\t%s\t%i\t%i\t%.2f" % (identifiers[x], identifiers[y], noverlap, ntotal, float(noverlap) / ntotal )

    pcoverages = map( lambda x: float(x) / lmali, coverages)    

    print "%s\tfailed\t" % prefix +\
          string.join(map( str, (nfragments, wmali,
                                 "%.2f" % (float(nfragments) / wmali),                                               
                                 nnon_overlapping, wmali * (wmali - 1) / 2,
                                 "%.2f" % (float(nnon_overlapping) / (wmali * (wmali - 1) / 2))
                                 )),"\t")

    print "%s\tpairs\t" % prefix +\
          string.join(map(str,
                          (min(pairs), max(pairs), "%.2f" % scipy.mean(pairs), scipy.median(pairs), "%.2f" % scipy.std(pairs))), "\t")
                                 
    print "%s\tppairs\t" % prefix +\
          string.join(map(lambda x: "%.2f" % x, (min(ppairs),
                                                 max(ppairs),
                                                 scipy.mean(ppairs),
                                                 scipy.median(ppairs),
                                                 scipy.std(ppairs))),"\t")
    print "%s\tcov\t" % prefix +\
          string.join( map(str, 
                           (min(coverages),
                            max(coverages),
                            "%.2f" % scipy.mean(coverages),
                            scipy.median(coverages),
                            "%.2f" % scipy.std(coverages))), "\t")
                                 
    print "%s\tpcov\t" % prefix +\
          string.join(map(lambda x: "%.2f" % x, (min(pcoverages),
                                                 max(pcoverages),
                                                 scipy.mean(pcoverages),
                                                 scipy.median(pcoverages),
                                                 scipy.std(pcoverages))), "\t")

    return fragments

##---------------------------------------------------------------------------------------
def GetIntronPositions( seq, gap_char = "-" ):
    """get intron positions."""

    last_upper = None

    transitions = []
    last_char = 0
    
    for x in range(len(seq)):
        
        if seq[x] == gap_char: continue

        last_char = x
        
        if last_upper == None:
            transitions.append(x)
        elif seq[x] in string.uppercase and not last_upper:
            transitions.append(x)
        elif seq[x] in string.lowercase and last_upper:
            transitions.append(x)
            
        last_upper = seq[x] in string.uppercase

    transitions.append( last_char )

    return transitions

##---------------------------------------------------------------------------------------
def MyMap( map, r, distance = 3, offset = 0):
    """returns
    -1, if no mapping possible with distance of 3 residues.
    0:  if mapping is out of bounds
    """
    if r < map.getRowFrom() or r > map.getRowTo():
        return 0

    b = min(1, r - distance)
    a = r
    while a >= b:
        c = map.mapRowToCol(a)
        if c != 0:
            return c + offset
        a -= 1

    b = max(map.getRowTo(), r + distance )
    a = r

    while a <= b:
        c = map.mapRowToCol(a)
        if c != 0: return c + offset
        a += 1

    return -1

def WriteGeneStructureCorrespondence( mali, identifiers, exons, param_master_pattern, gap_char = "-" , prefix = "" ):
    """split multiple alignment into clusters of orthologous transcripts.

    Orthologous transcripts are defined by similarity of gene structure to
    query sequences.

    Also: return matrix of gene structure compatibility

    0   : perfect compatibility (exact match)

    ratio of missed exon boundaries to total exon boundaries.
    
    100 : no compatibility
    """

    wmali = len(identifiers)
    lmali = len(mali[identifiers[0]])

    matrix_compatibility = numpy.zeros( (wmali, wmali) )

    if len(identifiers) == 0: return
    wmali = len(identifiers)
    lmali = len(mali[identifiers[0]])

    nok = 0
    nperfect   = 0

    ntotal_exons = 0
    nidentical_exons = 0
    nskipped_exons = 0

    ref_nok = 0
    ref_nperfect = 0

    ref_ntotal_exons = 0
    ref_nidentical_exons = 0
    ref_nskipped_exons = 0
    ref_ntotal = 0
    
    rx = re.compile( param_master_pattern )

    ## list of number of exons
    anexons = []

    ## exons in reference
    ref_nexons = 0
    x = 0
    for key1 in identifiers:

        seq = mali[key1]
        
        matches = []
        unassigned = []

        is_perfect = False

        anexons.append( len (exons[key1]) )
        if rx.search( key1 ):
            ref_nexons = len(exons[key1] )

        y = 0
        for key2 in identifiers:

            if key2 == key1: continue
            
            if param_loglevel >= 3:
                print "#############################################"
                print "# comparing %s to %s" % (key1, key2)
            
            mref = 0
            mcmp = 0
            
            seq_master = mali[key2]
            ref_exons = exons[key2]

            map_cmp2ref = MaliIO.getMapFromMali( seq, seq_master, gap_char )

            ## map exon boundaries to reference sequence
            cmp_exons = []

            if param_loglevel >= 5:
                print alignlib.writeAlignataTable(map_cmp2ref)
            
            for e in exons[key1]:
                ne = e.GetCopy()
                ne.mPeptideFrom = MyMap( map_cmp2ref, e.mPeptideFrom + 1, 3, -1 )
                ne.mPeptideTo   = MyMap( map_cmp2ref, e.mPeptideTo, 3, 0 )
                cmp_exons.append(ne)

            ## massage boundaries for terminal exons:
            if cmp_exons[0].mPeptideFrom <= 0: cmp_exons[0].mPeptideFrom = ref_exons[0].mPeptideFrom
            if cmp_exons[-1].mPeptideTo  <= 0: cmp_exons[-1].mPeptideTo  = ref_exons[-1].mPeptideTo
                
            if param_loglevel >= 4:
                for e in exons[key1]:
                    print "# exon", str(e)
                    
            if param_loglevel >= 3:                    
                for e in cmp_exons:
                    print "# exon", str(e)
                for e in ref_exons:
                    print "# exon", str(e)

            ## do exon comparison
            comparison = Exons.CompareGeneStructures( cmp_exons, 
                                                      ref_exons,
                                                      threshold_min_pide = 0,
                                                      threshold_slipping_exon_boundary = param_threshold_splipping_exon_boundary )


            if param_loglevel >= 3:
                print comparison.Pretty( prefix = "# EVAL: ")

            ## analyse results
            min_nexons = min(len(cmp_exons), len(ref_exons))                
            max_nexons = max(len(cmp_exons), len(ref_exons))

            similarity = (max_nexons - comparison.mNumIdenticalExons) * (abs( comparison.mNumDifferenceExons))
            
            is_perfect = False
            is_ok = False
            status = []

            # non-equivalent exon pairs
            ne = len(cmp_exons) - comparison.mNumIdenticalExons - comparison.mNumSkippedExons

            is_perfect = False
            is_ok = False
            if comparison.mNumIdenticalExons == 0:
                # F: complete and utter failure, no excuses
                status.append( "F" )
            else:
                if ne == 0:
                    # P: perfect conservation
                    status.append( "=" )
                    is_ok = True
                    is_perfect = True
                elif ne == min_nexons - comparison.mNumSkippedExons:
                    # D: completely different predictions
                    status.append( "D" )
                elif ne in (1,2):
                    # A: almost conserved
                    status.append( "A" )
                    is_ok = True
                elif ne > 2:
                    # M : mostly conserved (in case of long proteins that is good enough).
                    if (100 * comparison.mNumIdenticalExons) / max_nexons > param_evaluate_min_percent_exon_identity:
                        status.append( "M" )
                    else:
                    # S : spuriously conserved
                        status.append( "S" )                    
                else:
                    # U: unconserved
                    status.append( "U" )

            if len(cmp_exons) > len(ref_exons):
                status.append( ">" )
            elif len(ref_exons) < len(cmp_exons):
                status.append( "<" )
            else:
                status.append( "=" )

            if min_nexons == max_nexons and min_nexons == 1:
                status.append( "S" )
            elif min_nexons == 1 and max_nexons == 2:
                status.append( "s")
            elif min_nexons == 2 and max_nexons == 2:
                status.append( "D" )
            elif min_nexons == 2 and max_nexons > 2:
                status.append( "d" )
            elif min_nexons == max_nexons:
                status.append( "M" )
            elif min_nexons > 2 and max_nexons > 2:
                status.append( "m" )                
            else:
                status.append( "U")
                
            status = string.join( status, "")

            structure_compatibility = 100
            
            if is_ok:
                nok += 1
                structure_compatibility = 100 - 100 * (comparison.mNumIdenticalExons + comparison.mNumSkippedExons) / len(cmp_exons)
            if is_perfect:
                nperfect += 1
                structure_compatibility = 0
                
            if abs(comparison.mNumDifferenceExons) > param_max_exons_difference:
                compatibility_value = 100
            else:
                compatibility_value = structure_compatibility

            t = comparison.mNumRefBoundaries + comparison.mNumCmpBoundaries

            if t == 0:
                compatibility_value = 0
            else:
                compatibility_value = 100 * (comparison.mNumMissedRefBoundaries + comparison.mNumMissedCmpBoundaries) / t
                
            matrix_compatibility[x][y] = compatibility_value
            
            nidentical_exons += comparison.mNumIdenticalExons
            nskipped_exons   += comparison.mNumSkippedExons
            ntotal_exons     += len(cmp_exons)

            if param_loglevel >= 2:
                print "%s\tgenepair\t%s\t%s\t%s\t%i\t%i\t%i\t%s" % (prefix, key1, key2, status, compatibility_value,
                                                                    len(cmp_exons), len(ref_exons), str(comparison))

            ## comparison to reference: count separately:
            if rx.search( key2 ):
                ref_nidentical_exons += comparison.mNumIdenticalExons
                ref_nskipped_exons   += comparison.mNumSkippedExons
                ref_ntotal_exons     += len(cmp_exons)
                if is_ok: ref_nok += 1
                if is_perfect: ref_nperfect += 1
                ref_ntotal += 1

            y += 1
            
        x += 1
                
    ntotal = wmali * ( wmali - 1)

    print "%s\tallstructure\t%i\t%i\t%i\t%6.4f\t%6.4f\t%i\t%i\t%i\t%6.4f\t%6.4f" % (prefix,
                                                                                    ntotal, nperfect, nok,
                                                                                    float(nperfect) / ntotal, float(nok) / ntotal,
                                                                                    ntotal_exons, nidentical_exons, nskipped_exons, 
                                                                                    float(nidentical_exons) / ntotal_exons,
                                                                                    float(nidentical_exons + nskipped_exons) / ntotal_exons)

    if ref_ntotal > 0:
        if ref_ntotal_exons == 0:
            raise "no exons in reference : ref_ntotal_exons = 0, ref_ntotal = %i" % (ref_ntotal)
        
        print "%s\trefstructure\t%i\t%i\t%i\t%6.4f\t%6.4f\t%i\t%i\t%i\t%6.4f\t%6.4f" % (prefix,
                                                                                        ref_ntotal, ref_nperfect, ref_nok,
                                                                                        float(ref_nperfect) / ref_ntotal, float(ref_nok) / ref_ntotal,
                                                                                        ref_ntotal_exons, ref_nidentical_exons, ref_nskipped_exons, 
                                                                                        float(ref_nidentical_exons) / ref_ntotal_exons,
                                                                                        float(ref_nidentical_exons + ref_nskipped_exons) / ref_ntotal_exons)
        
    print "%s\tnexons\t%i\t%i\t" % (prefix,
                                    len(anexons), ref_nexons) +\
                                    string.join(map(lambda x: "%.2f" % x, (min(anexons),
                                                                           max(anexons),
                                                                           scipy.mean(anexons),
                                                                           scipy.median(anexons),
                                                                           scipy.std(anexons))), "\t")

    return matrix_compatibility

##---------------------------------------------------------------------------------------                                                         
def oldClusterMatrixClosestDistance( matrix_weights,
                                  identifiers,
                                  master_pattern,
                                  species_pattern,
                                  matrix_compatibility = None):
    """cluster identifiers by their coverage.

    """

    rxm = re.compile( master_pattern )
    rxs = re.compile( species_pattern )

    # positions of masters in identifier list
    master_ids = []
    # members of each clusters
    clusters= {}
    # set of species per cluster
    species = {}
    # unassigned identifiers (no masters)
    unassigned = {}

    # initialize
    for x in range(len(identifiers)):
        i = identifiers[x]
        g = rxs.search( i )
        if g:
            s = g.groups()[0]
            species[i] = {}
            
        if rxm.search( i ):
            master_ids.append( x )
            clusters[i] = []
        else:
            unassigned[i] = 1

    if len(master_ids) == 0:
        clusters = { None : identifiers }
        return clusters, []

    ## merge clusters
    skip = {}
    new = []
    for x in range(0, len(master_ids)-1):
        xm = master_ids[x]
        xi = identifiers[xm]
        if xi in clusters:
            new.append( xm )            
            for y in range(x+1, len(master_ids)):
                ym = master_ids[y]
                yi = identifiers[ym]
                if yi not in clusters: continue
                if matrix_compatibility[xm][ym] <= param_cluster_merging_max_distance:
                    if param_loglevel >= 3:
                        print "# merging %s with %s" % (xi, yi )
                    clusters[xi] += clusters[yi] + [yi]
                    del clusters[yi]

    master_ids = new

    # build edges
    edges = []
    best_matches = {}
    # if masters are identical, the first one gets all the
    # the members. 
    for x in range(len(identifiers)):
        
        i = identifiers[x]
        g = rxs.search( i )
        if g:
            s = g.groups()[0]
        else:
            s = None
            
        if i in clusters: continue
        
        for m in master_ids:
            
            c = matrix_weights[x][m]
            
            if matrix_compatibility:
                co = max(matrix_compatibility[x][m], matrix_compatibility[m][x])
            else:
                co = 0
                
            if c <= param_clustering_max_distance and co <= param_max_compatibility:
                edges.append( (c, x, m, s, co) )
                    

        # best: best entry per identifier with compatible gene structure
        best = None
        best_m = None

        if param_loglevel >= 5:
            print "# pair:", i, identifiers[best_m], matrix_weights[x][best_m]
            
        for m in master_ids:
            if param_loglevel >= 5:
                print "# pair:", i, identifiers[m], matrix_weights[x][m]

            if matrix_compatibility:
                co = max(matrix_compatibility[x][m], matrix_compatibility[m][x])
            else:
                co = 0

            if matrix_compatibility and co <= param_max_compatibility:
                if best == None or matrix_weights[x][m] < best:
                    best = matrix_weights[x][m]
                    best_m = m

        if best != None and best <= param_clustering_max_distance:
            best_matches[i] = (best_m, best)
            
    # go through edge list by increasing distance and add identifiers
    # to clusters where they don't have another member of the same species.
    edges.sort()
    for coverage, x, m, s, compatibility in edges:

        cm = identifiers[m]
        i = identifiers[x]

        if i not in unassigned: continue
        
        if s and s not in species[cm]:
            species[cm][s] = 1
            clusters[cm].append( i )
            del unassigned[i]
            if param_loglevel >= 4:
                print "# assigning %s to %s with weight %f and compatibility %f" % ( i, cm, coverage, compatibility )

    ## assign remaining to best cluster
    for i in unassigned.keys():
        if i in best_matches:
            bm, coverage = best_matches[i]
            cm = identifiers[bm]
            if param_loglevel >= 4:            
                print "# assigning %s to %s as best match with weight %f" % ( i, cm, coverage )            
            clusters[cm].append( i )
            del unassigned[i]
            
    
    # assign masters to clusters. If they have no members,
    # assign to best match
    reps = []
    mems = []
    for m in master_ids:
        i = identifiers[m]
        if len(clusters[i]) > 0:
            clusters[i].append( i )
            reps.append( m )
        else:
            mems.append( m )

    for x in mems:
        i = identifiers[x]
        best = matrix_weights[x][reps[0]]
        best_m = reps[0]
        for m in reps[1:]:
            if matrix_weights[x][m] < best:
                best = matrix_weights[x][m]
                best_m = m

        if best <= param_clustering_max_distance:
            if param_loglevel >= 4:
                print "# assigning %s to %s with weight %f" % ( i, identifiers[best_m], best)
            clusters[identifiers[best_m]].append(i)
            del clusters[i]
        else:
            unassigned[i] = 1
            clusters[i].append( i )

#     # if two masters are identical to another, merge
#     skip = {}
#     for x in range(0, len(reps)-1):
#         xi = reps[x]
#         for y in range( x+1, len(reps)):
#             yi = reps[y]
#             if y in skip: continue
#             if matrix_weights[xi][yi] <= param_cluster_merging_max_distance:
#                 if param_loglevel >= 3:
#                     print "# merging %s with %s" % (identifiers[xi], identifiers[yi] )
#                 clusters[identifiers[xi]] += clusters[identifiers[yi]]
#                 del clusters[identifiers[yi]]
#                 skip[y] = 1
            
    return clusters, unassigned.keys()

##---------------------------------------------------------------------------------------                                                         
def ClusterMatrixClosestDistance( matrix_weights,
                                  identifiers,
                                  master_pattern,
                                  species_pattern,
                                  matrix_compatibility = None):
    """cluster identifiers by their coverage.

    """

    rxm = re.compile( master_pattern )
    rxs = re.compile( species_pattern )

    # positions of masters in identifier list
    master_ids = []
    # members of each clusters
    clusters= {}
    # set of species per cluster
    species = {}
    # unassigned identifiers (no masters)
    unassigned = {}

    # initialize, each cluster contains one master
    for x in range(len(identifiers)):
        i = identifiers[x]
        g = rxs.search( i )
        if g:
            s = g.groups()[0]
            species[i] = {}
            
        if rxm.search( i ):
            master_ids.append( x )
            clusters[i] = [ (i, x, "", 0.0, True) ] 
        else:
            unassigned[i] = 1

    if len(master_ids) == 0:
        clusters = { None : identifiers }
        return clusters, []

    if param_loglevel >= 2:
        print "# CLUSTERING: at start: %i/%i clusters for %i unassigned" % (len(clusters), len(master_ids), len(unassigned))
        
    ## merge clusters if masters are similar
    skip = {}
    new = []
    for x in range(0, len(master_ids)):
        xm = master_ids[x]
        xi = identifiers[xm]
        if xi in clusters:
            new.append( xm )            
            for y in range(x+1, len(master_ids)):
                if y == x: continue
                ym = master_ids[y]
                yi = identifiers[ym]
                if yi not in clusters: continue
                if matrix_weights[xm][ym] <= param_cluster_merging_max_distance:
                    if param_loglevel >= 3:
                        print "# merging %s with %s" % (xi, yi )
                    clusters[xi] += clusters[yi]
                    del clusters[yi]

    if param_loglevel >= 2:
        print "# CLUSTERING: after merging: %i clusters for %i unassigned" % (len(clusters), len(unassigned))

    master_ids = new

    # assign matches to best cluster
    for x in range(len(identifiers)):
        
        id = identifiers[x]
        g = rxs.search( id )
        if g:
            species = g.groups()[0]
        else:
            species = None
            
        if id not in unassigned: continue
        
        # best: best entry per identifier with compatible gene structure
        best = None
        best_m = None

        for m in master_ids:
            if param_loglevel >= 5:
                print "# pair:", id, identifiers[m], matrix_weights[x][m]

            if best == None or matrix_weights[x][m] < best:
                best = matrix_weights[x][m]
                best_m = m
        
        if best != None and best <= param_clustering_max_distance:
            if param_loglevel >= 3:
                print "# assigning %s to %s with weight %f" % (id, identifiers[best_m], best)
            clusters[identifiers[best_m]].append( (id, x, species, best, False) )
            del unassigned[id]

    if param_loglevel >= 2:
        print "# CLUSTERING: after assignment: %i clusters for %i unassigned" % (len(clusters), len(unassigned))

    # for each cluster sort according to compatibility and keep best
    for m in master_ids:

        new = []
        to_sort = []
        master_id = identifiers[m]
        for id, index, species, weight, is_master in clusters[master_id]:
            if is_master:
                new.append( id )
            else:
                to_sort.append( (species, matrix_compatibility[index][m], weight, id) )

        # this sorts first by species and then by compatibility in increasing order
        to_sort.sort()
            
        last_species = None
        
        for species, compatibility, weight, id in to_sort:

            if last_species == species:
                if param_loglevel >= 3:
                    print "# cluster: %s: removing %s at weight %f and compatibility %f" % (master_id, id, weight, compatibility)
                unassigned[id] = 1                
                continue
            else:
                if param_loglevel >= 3:
                    print "# cluster: %s: keeping %s at weight %f and compatibility %f" % (master_id, id, weight, compatibility)
                
            last_species = species
            new.append( id )
            
        clusters[master_id] = new

    if param_loglevel >= 2:
        print "# CLUSTERING: after compatiblitity: %i clusters for %i unassigned" % (len(clusters), len(unassigned))
        
    return clusters, unassigned.keys()


##---------------------------------------------------------------------------------------
def WriteSpeciesCoverage( identifiers, species_pattern, prefix = ""):
    """write number of species present in identifiers."""

    species = {}
    nunknown = 0
    rx = re.compile( species_pattern )
    
    for i in identifiers:
        x = rx.search(i)
        if x:
            s = x.groups()[0]
        else:
            nunknown += 1
            s = "unknown"
            
        if s not in species: species[s] = 0
        species[s] += 1

    if len(species) == 0:
        max_species = 0
    else:
        max_species = max(species.values())
    print "%s\tspecies\t%i\t%i\t%i\t%i" % (prefix,
                                           len(species),
                                           max_species,
                                           len(filter( lambda x: x >= 2, species.values())),
                                           nunknown )


##---------------------------------------------------------------------------------------
def WriteCodonSummary( mali, identifiers, frame_columns, prefix = "", gap_char = "-"):
    """write codon summary."""
    
    new_mali = {}
    aligned = []
    codons = []
    stops = []
    nclean = 0
    total_no_stops = 0
    for key, seq in core_mali.items():
        new_mali[key], naligned, ncodons, nstops = MaliIO.getCodonSequence( seq, frame_columns, param_gap_char, remove_stops = True)
        aligned.append( naligned )
        codons.append( ncodons )
        stops.append( nstops )
        if nstops == 0:
            total_no_stops += 1
        if naligned == ncodons and nstops == 0:
            nclean += 1
            
    print "%s\tcodons\t%i\t%i\t" % (prefix, nclean, total_no_stops) +\
          string.join(map(lambda x: "%.2f" % x, (min(aligned),
                                                 max(aligned),
                                                 scipy.mean(aligned),
                                                 scipy.median(aligned),
                                                 scipy.std(aligned))),"\t") + "\t" +\
          string.join(map(lambda x: "%.2f" % x, (min(codons),
                                                 max(codons),
                                                 scipy.mean(codons),
                                                 scipy.median(codons),
                                                 scipy.std(codons))),"\t") + "\t" +\
          string.join(map(lambda x: "%.2f" % x, (min(stops),
                                                 max(stops),
                                                 scipy.mean(stops),
                                                 scipy.median(stops),
                                                 scipy.std(stops))),"\t")
                                                 
        
    return new_mali

##---------------------------------------------------------------------------------------
def WriteRadius( mali, identifiers, prefix = "", gap_char = "-"):
    """write percent identities in pairwise comparisons both for nucleotide acids and amino acids."""

    pides_na = []
    seq_aa = []
    for x in range(0,len(identifiers)):

        seq_aa.append( Genomics.TranslateDNA2Protein(mali[identifiers[x]]))
                       
        for y in range(x+1, len(identifiers)):
            if x == y: continue
            pides_na.append( MaliIO.getPercentIdentity( mali[identifiers[x]], mali[identifiers[y]], gap_char) )

    pides_aa = []
    for x in range(0,len(identifiers)-1):
        for y in range(x+1, len(identifiers)):        
            pides_aa.append( MaliIO.getPercentIdentity( seq_aa[x], seq_aa[y], gap_char) )            
            
    print "%s\tpide\t%i\t" % (prefix, len(pides_na)) +\
          string.join(map(lambda x: "%.2f" % x, (min(pides_na),
                                                 max(pides_na),
                                                 scipy.mean(pides_na),
                                                 scipy.median(pides_na),
                                                 scipy.std(pides_na))),"\t") + "\t" +\
          string.join(map(lambda x: "%.2f" % x, (min(pides_aa),
                                                 max(pides_aa),
                                                 scipy.mean(pides_aa),
                                                 scipy.median(pides_aa),
                                                 scipy.std(pides_aa))),"\t") 
    
            
##------------------------------------------------------------
if __name__ == '__main__':

    try:
        optlist, args = getopt.getopt(sys.argv[1:], param_short_options, param_long_options)
    except getopt.error, msg:
        print USAGE, msg
        sys.exit(2)

    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "-h", "--help" ):
            print USAGE
            sys.exit(0)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ("-o", "--file-output"):
            param_filename_output = a
        elif o == "--min-overlap-residues":
            param_min_overlap_residues = int(a)
        elif o == "--min-overlap-percent":
            param_min_overlap_percent = int(a)
        elif o == "--min-coverage-residues":
            param_min_coverage_residues = int(a)
        elif o == "--min-coverage-percent":
            param_min_coverage_percent = int(a)
        elif o in ("-m", "--master-pattern"):
            param_master_pattern = a
        elif o in ("-s", "--species-pattern"):
            param_species_pattern = a
        elif o in ("-e", "--exons"):
            param_filename_exons = a
        elif o in ("-c", "--cluster"):
            param_do_cluster = True
        elif o in ("-f", "--remove-fragments"):
            param_remove_fragments = True
        elif o in ("-p", "--prefix"):
            param_prefix = a
        elif o == "--components":
            param_filename_components = a

    print E.GetHeader()
    print E.GetParams()
            
    ## 1. read multiple alignment in fasta format
    all_mali, all_identifiers = MaliIO.readFasta( sys.stdin )

    if len(all_identifiers) == 0:
        raise "alignment is empty."

    if param_loglevel >= 1:
        print "# read mali with %i entries." % len(all_identifiers)

    if param_filename_components:

        infile = open(param_filename_components, "r")
        components = {}
        for line in infile:
            if line[0] == "#": continue
            if line[0] == ">": continue
            a, b = line[:-1].split("\t")[:2]
            if b not in components:
                components[b] = []
            components[b].append(a)
        
        if param_loglevel >= 1:
            print "# read %i components." % len(components)
    else:
        components = { 'all': all_identifiers }

    if param_filename_exons:
        exons = Exons.ReadExonBoundaries( open(param_filename_exons, "r"), filter=all_mali )
        if param_loglevel >= 2:
            print "# read %i exons." % len(exons)
    else:
        exons = {}

    print "# PREFIX\tsummary\tNSEQUENCES\tNASSIGNED\tNCLUSTERS\tNASSIGNED\tUNASSIGNED"
    print "# PREFIX\tcluster\tNMEMBERS\tMEMBERS"
    print "# PREFIX\tfragments\tNFRAGMENTS\tFRAGMENTS"

    print "# PREFIX\tpide\tNPAIRS\tNAMIN\tNAMAX\tNAMEAN\tNAMEDIAN\tNASTDDEV\tAAMIN\tAAMAX\tAAMEAN\tAAMEDIAN\tAASTDDEV"
    
    print string.join( ("# PREFIX", "codons", 
                        "NCLEAN","NNOSTOPS",
                        "ALIGNED_MIN","ALIGNED_MAX","ALIGNED_MEAN","ALIGNED_MEDIAN","ALIGNED_STDDEV",
                        "CODONS_MIN","CODONS_MAX","CODONS_MEAN","CODONS_MEDIAN","CODONS_STDDEV",
                        "STOPS_MIN","STOPS_MAX","STOPS_MEAN","STOPS_MEDIAN","STOPS_STDDEV"), "\t")

    print string.join( ("# PREFIX", "allstructure", 
                        "NTOTAL","NPERFECT", "NOK",
                        "PPERFECT", "POK",
                        "NTOTAL_EXONS", "NIDENTICAL_EXONS", "NSKIPPED_EXONS",
                        "PIDENTICAL_EXONS", "PIDENTICAL_EXONS"), "\t")

    print string.join( ("# PREFIX", "refstructure", 
                        "NTOTAL","NPERFECT", "NOK",
                        "PPERFECT", "POK",
                        "NTOTAL_EXONS", "NIDENTICAL_EXONS", "NSKIPPED_EXONS",
                        "PIDENTICAL_EXONS", "PIDENTICAL_EXONS"), "\t")

    print string.join( ("# PREFIX", "exons", 
                        "NDATA","NREF_EXONS",
                        "NEXONS_MIN","NEXONS_MAX","NEXONS_MEAN","NEXONS_MEDIAN","NEXONS_STDDEV"), "\t")

    print string.join( ("# PREFIX", "species", 
                        "NSPECIES","SPECIES_MAX",
                        "MAX_PER_SPECIES","UNKNOWN"), "\t")

    print string.join( ("# PREFIX", "failed",
                        "NFAILED_SEQS","NTOTAL_SEQS", "PFAILED_SEQS",
                        "NFAILED_PAIRS","NTOTAL_PAIRS", "PFAILED_PAIRS"), "\t")

    print string.join( ("# PREFIX", "npairs",
                        "NPAIRS_MIN","NPAIRS_MAX","NPAIRS_MEAN","NPAIRS_MEDIAN","NPAIRS_STDDEV"), "\t")
                        
    print string.join( ("# PREFIX", "ppairs",
                        "PPAIRS_MIN","PPAIRS_MAX","PPAIRS_MEAN","PPAIRS_MEDIAN","PPAIRS_STDDEV"), "\t")

    print string.join( ("# PREFIX", "cov",
                        "COV_MIN","COV_MAX","COV_MEAN","COV_MEDIAN","COV_STDDEV"), "\t")

    print string.join( ("# PREFIX", "pcov",
                        "PCOV_MIN","PCOV_MAX","PCOV_MEAN","PCOV_MEDIAN","PCOV_STDDEV"), "\t")

    print string.join( ("# PREFIX", "genepair",
                        "STATUS", "COMPATIBILITY", "CMP_NEXONS", "REF_NEXONS", Exons.ComparisonResult().GetHeader()), "\t")
                        
    for key, identifiers in components.items():

        prefix = "%s%s_all" % (param_prefix, key)
        
        ## 1. remove gaps in multiple alignment
        mali = MaliIO.removeGappedColumns( MaliIO.getSubset( all_mali, identifiers), param_gap_char )

        ## 1. remove gaps in multiple alignment
        if param_do_cluster:
            matrix_overlap, matrix_total, coverages = GetOverlapMatrix( mali, identifiers)

            matrix_coverage = 100 - ((100 * matrix_overlap) / matrix_total)

            if param_loglevel >= 3:
                print "# overlap\n", matrix_overlap
                print "# total\n", matrix_total
                print "# coverage\n", matrix_coverage
                print "# coverages\n", coverages

            matrix_compatibility = WriteGeneStructureCorrespondence( mali, identifiers, exons, param_master_pattern, prefix = prefix )

            if param_loglevel >= 3:
                print "# compatibility\n", matrix_compatibility

            clusters, unassigned = ClusterMatrixClosestDistance( matrix_coverage, identifiers,
                                                                 param_master_pattern, param_species_pattern,
                                                                 matrix_compatibility)
        else:
            clusters = { 'all': identifiers }

        cluster_id = 0

        print "%s%s\tsummary\t%i\t%i\t%i\t%i\t%s" % (param_prefix, key,
                                                     len(all_identifiers),
                                                     len(all_identifiers) - len(unassigned),
                                                     len(clusters),
                                                     len(unassigned),                                                
                                                     string.join(unassigned, "\t"))

        for k, v in clusters.items():

            cluster_id += 1
            prefix = "%s%s_%i" % (param_prefix, key, cluster_id)
            
            if param_loglevel >= 1:
                print "###########################################################"

            if len(v) == 0:
                print "# cluster %s contains no sequence" % (prefix)
                continue
                
            if len(v) == 1:
                print "# cluster %s contains only one sequence %s" % (prefix, v[0])
                continue
                
            # 1. remove gaps in multiple alignment
            ungapped_mali = MaliIO.removeGappedColumns( MaliIO.getSubset( mali, v ), param_gap_char )

            # write overlap parameters
            fragments = WriteOverlap( ungapped_mali, v, prefix = prefix )

            not_fragments = filter( lambda x: x not in fragments, v )

            print "%s\tcluster\t%s\t%i\t%s" % (prefix, k, len(not_fragments), string.join(not_fragments, "\t"))

            if len(fragments) > 0:
                print "%s\tfragments\t%i\t%s" % (prefix, len(fragments), string.join(fragments, "\t"))

            if len(not_fragments) == 1:
                continue

            if param_remove_fragments:
                core_mali = MaliIO.removeGappedColumns( MaliIO.getSubset( ungapped_mali, not_fragments ), param_gap_char )
            else:
                core_mali = ungapped_mali

            # check how many exons are recovered.
            WriteGeneStructureCorrespondence( core_mali, not_fragments, exons, param_master_pattern, prefix = prefix )

            # write coverage of genomes
            WriteSpeciesCoverage( not_fragments, species_pattern = param_species_pattern, prefix = prefix )

            # get frame columns
            frame_columns = MaliIO.getFrameColumnsForMasterPattern( core_mali, not_fragments, param_master_pattern, param_gap_char )

            # translate multiple alignment
            codon_mali = WriteCodonSummary( core_mali, not_fragments, frame_columns, prefix, param_gap_char )
            
            # write coverage of genomes
            WriteRadius( codon_mali, not_fragments, prefix = prefix )

            if param_output_pattern:
                filename = param_output_pattern % cluster_id
                outfile = open( filename, "w")
                for id in core_mali.keys():
                    outfile.write( ">%s\n%s\n" % (id, core_mali[id]))
                outfile.close()

    print E.GetFooter()
    
    



################################################################################
#   Gene prediction pipeline 
#
#   $Id: Prediction.py 2781 2009-09-10 11:33:14Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
Prediction.py - Gene prediction with exonerate/genewise
========================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

"""
import os, sys, string, re, getopt, tempfile, copy

import Genomics
import alignlib

## number of nucleotides to ignore for counting stop-codons
## This used to be three, but is now set to 0 in order to keep
## accounting for stop codons consistent with the disruptions
## table.
GlobalBorderStopCodons = 0
GlobalStopCodons = ("TAG", "TAA", "TGA")

class Error(Exception):
    """Base class for exceptions in this module."""
    def __str__(self):
        return str(self.message)
    def _get_message(self, message): return self._message
    def _set_message(self, message): self._message = message
    message = property(_get_message, _set_message)

class InputError(Error):
    """Exception raised for errors in the input.

    Attributes:
        expression -- input expression in which the error occurred
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class AlignmentError(Error):
    """Exception raised by inconsistent alignments.

    Attributes:
        expression -- input expression in which the error occurred
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message


class Prediction:
    
    def __init__(self, expand = 1):

        self.mExpand = expand
        
        self.mPredictionId = 0
        self.mQueryToken = 0
        self.mQueryFrom = 0
        self.mQueryTo = 0
        self.mSbjctToken = 0
        self.mSbjctStrand = 0
        self.mSbjctFrom = 0
        self.mSbjctTo = 0
        self.mRank = 0
        self.score = 0
        self.mQueryLength = 0
        self.mQueryCoverage = 0
        self.mNGaps = 0
        self.mNFrameShifts = 0
        self.mNIntrons = 0
        self.mNSplits = 0
        self.mNStopCodons = 0
        self.mPercentIdentity = 0
        self.mPercentSimilarity = 0
        self.mTranslation = ""
        self.mSbjctGenomeFrom = 0
        self.mSbjctGenomeTo = 0
        self.mAlignmentString = ""
        self.mQueryAli = ""
        self.mSbjctAli = ""
        
        if self.mExpand:
            self.mMapPeptide2Translation = alignlib.makeAlignmentVector()
            self.mMapPeptide2Genome = []
        else:
            self.mMapPeptide2Translation = None
            self.mMapPeptide2Genome = None
        self.mNAssembled = 0

    def __del__(self):
        if self.mExpand:
            self.mMapPeptide2Translation.clear()
            self.mMapPeptide2Genome = []
        self.mTranslation = ""
        self.mAlignmentString = ""
        
    def __eq__(self, other ):

        return self.mQueryToken == other.mQueryToken and \
               self.mSbjctToken == other.mSbjctToken and \
               self.mSbjctStrand == other.mSbjctStrand and \
               self.score == other.score and \
               self.mQueryFrom == other.mQueryFrom and \
               self.mQueryTo == other.mQueryTo and \
               self.mSbjctGenomeFrom == other.mSbjctGenomeFrom and \
               self.mSbjctGenomeTo == other.mSbjctGenomeTo and \
               self.mAlignmentString == other.mAlignmentString 

    def cleanAlignmentInfo( self ):
        """remove alignment information (to save space)."""
        self.mTranslation, self.mAlignmentString, self.mQueryAli, self.mSbjctAli = None, None, None, None
        
    def getCopy( self ):
        """return a new copy.
        """

        new_entry = Prediction()

        new_entry.mExpand = self.mExpand 
        
        new_entry.mPredictionId = self.mPredictionId 
        new_entry.mQueryToken = self.mQueryToken 
        new_entry.mQueryFrom = self.mQueryFrom 
        new_entry.mQueryTo = self.mQueryTo 
        new_entry.mSbjctToken = self.mSbjctToken 
        new_entry.mSbjctStrand = self.mSbjctStrand 
        new_entry.mSbjctFrom = self.mSbjctFrom 
        new_entry.mSbjctTo = self.mSbjctTo 
        new_entry.mRank = self.mRank 
        new_entry.score = self.score 
        new_entry.mQueryLength = self.mQueryLength 
        new_entry.mQueryCoverage = self.mQueryCoverage 
        new_entry.mNGaps = self.mNGaps 
        new_entry.mNFrameShifts = self.mNFrameShifts 
        new_entry.mNIntrons = self.mNIntrons 
        new_entry.mNSplits = self.mNSplits 
        new_entry.mNStopCodons = self.mNStopCodons 
        new_entry.mPercentIdentity = self.mPercentIdentity 
        new_entry.mPercentSimilarity = self.mPercentSimilarity 
        new_entry.mTranslation = self.mTranslation 
        new_entry.mSbjctGenomeFrom = self.mSbjctGenomeFrom 
        new_entry.mSbjctGenomeTo = self.mSbjctGenomeTo 
        new_entry.mAlignmentString = self.mAlignmentString 
        new_entry.mQueryAli = self.mQueryAli 
        new_entry.mSbjctAli = self.mSbjctAli 

        if self.mExpand:
            new_entry.mMapPeptide2Translation = alignlib.makeAlignmentVector()
            alignlib.copyAlignment( new_entry.mMapPeptide2Translation, self.mMapPeptide2Translation)
            new_entry.mMapPeptide2Genome = Genomics.String2Alignment( new_entry.mAlignmentString) 
        else:
            new_entry.mMapPeptide2Translation = self.mMapPeptide2Translation = None
            new_entry.mMapPeptide2Genome = self.mMapPeptide2Genome = None

        return new_entry

    def Expand( self ):
        self.mExpand = True
        
        if self.mMapPeptide2Translation.getLength() > 0:
            f = alignlib.AlignmentFormatEmissions( self.mMapPeptide2Translation )
            self.mQueryAli, self.mSbjctAli = f.mRowAlignment, f.mColAlignment
            self.mQueryFrom = self.mMapPeptide2Translation.getRowFrom()
            self.mQueryTo = self.mMapPeptide2Translation.getRowTo()
            self.mSbjctFrom = self.mMapPeptide2Translation.getColFrom()
            self.mSbjctTo = self.mMapPeptide2Translation.getColTo()

        self.mMapPeptide2Genome = Genomics.String2Alignment( self.mAlignmentString )

    def Add( self, const_other,
             combine_contig = False,
             allow_overlap = False,
             contig_size = 0,
             combine_queries = False,
             as_intron = False ):
        """add one entry to another.

        This procedure allows to add
        
        - predictions on different contigs if combine_contig = True
        - overlapping predictions on the same query if allow_overlap = True
        - results from different queries if combine_queries = True

        - if as_intron is set to true, the new fragment is added as an intron.
        
        """

        ## create working copies of each prediction
        other = const_other.getCopy()
        this  = self.getCopy()

        other.Expand()
        this.Expand()

        if as_intron:
            code = "I"
        else:
            code = "P"

        ## check for query overlaps
        if this.mQueryToken == other.mQueryToken:

            query_overlap = max( 0, min(this.mQueryTo, other.mQueryTo) -\
                                 max(this.mQueryFrom, other.mQueryFrom) + 1)

            if query_overlap > 0:

                if allow_overlap:
                    overlap = query_overlap
                    ## if queries overlap, truncate this before adding the other
                    this.mMapPeptide2Translation.removeRowRegion( this.mQueryTo - overlap + 1, this.mQueryTo )
                    other.mMapPeptide2Translation.moveAlignment( 0, -overlap )
                    this.mQueryTo -= overlap
                    this.mTranslation = this.mTranslation[:-overlap]

                    ## remove aligned residues from the back
                    for x in range(len(this.mMapPeptide2Genome) - 1, 0, -1):
                        if this.mMapPeptide2Genome[x][1] <= overlap:
                            overlap -= this.mMapPeptide2Genome[x][1]
                            del this.mMapPeptide2Genome[x]
                        else:
                            break
                    this.mMapPeptide2Genome[-1] = (this.mMapPeptide2Genome[-1][0],
                                                   this.mMapPeptide2Genome[-1][1] - overlap,
                                                   this.mMapPeptide2Genome[-1][2] - overlap * 3)
                else:
                    raise ValueError, "refusing to add overlapping entries: overlap = %i, queries:\n%s\n%s\n, set allow_overlap = True " % (query_overlap, str(this), str(other))


        else:
            if not combine_queries:
                raise ValueError, "refusing to add different queries - set combine_queries = True."

        if this.mSbjctToken != other.mSbjctToken or \
               this.mSbjctStrand != other.mSbjctStrand :
            if combine_contig:
                this.mSbjctToken += "-" + other.mSbjctToken
                this.mSbjctStrand += other.mSbjctStrand
            else:
                raise ValueError, "can not add different sbjct."                

        sbjct_overlap = max(0, min(this.mSbjctGenomeTo, other.mSbjctGenomeTo) -\
                            max(this.mSbjctGenomeFrom, other.mSbjctGenomeFrom), 0)

        if sbjct_overlap > 0:
            if not combine_contig:
                raise ValueError, "refusing to add overlapping entries: overlap = %i, sbjct:\n%s\n%s\n" % (sbjct_overlap, str(this), str(other))

        if this.mSbjctToken == other.mSbjctToken:

            ## set precedence
            if this.mSbjctGenomeFrom < other.mSbjctGenomeFrom:
                first = this
                second = other
            else:
                first = other
                second = this

            ## get length of gap
            d_na = second.mSbjctGenomeFrom - first.mSbjctGenomeTo

            if this.mQueryToken != other.mQueryToken:
                d_aa = first.mQueryLength - first.mQueryTo                 
                # create a new virtual query by concatenating
                # the two queries
                this.mQueryToken += "-" + other.mQueryToken

                # sort out the alignment
                second.mMapPeptide2Translation.moveAlignment( first.mQueryLength, 0 )

                this.mQueryLength = first.mQueryLength + second.mQueryLength

            else:
                d_aa = second.mQueryFrom - first.mQueryTo - 1
            
            this.mSbjctGenomeFrom = min(this.mSbjctGenomeFrom, other.mSbjctGenomeFrom )
            this.mSbjctGenomeTo = max(this.mSbjctGenomeTo, other.mSbjctGenomeTo )        

            this.mMapPeptide2Genome = first.mMapPeptide2Genome + [(code, d_aa, d_na)] + second.mMapPeptide2Genome
            this.mTranslation = first.mTranslation + second.mTranslation

            second.mMapPeptide2Translation.moveAlignment( 0, first.mSbjctTo - 1 )
            
        else:
            ## join on different contigs
            d_na = contig_size - this.mSbjctGenomeTo + other.mSbjctGenomeFrom + query_overlap * 3
            d_aa = other.mQueryFrom - this.mQueryTo - 1
            this.mMapPeptide2Genome += [(code, d_aa, d_na),] + other.mMapPeptide2Genome
            this.mTranslation += other.mTranslation 
            other.mMapPeptide2Translation.moveAlignment( 0, this.mSbjctTo - 1 )

            this.mSbjctGenomeFrom = this.mSbjctGenomeFrom
            this.mSbjctGenomeTo = contig_size + other.mSbjctGenomeTo

        ## now fill self from first and this
        self.mQueryToken = first.mQueryToken
        self.mQueryLength = this.mQueryLength
        
        nthis  = this.mMapPeptide2Translation.getLength() - this.mMapPeptide2Translation.getNumGaps()
        nother = other.mMapPeptide2Translation.getLength() - other.mMapPeptide2Translation.getNumGaps()

        self.mMapPeptide2Genome = first.mMapPeptide2Genome
        self.mSbjctGenomeFrom = this.mSbjctGenomeFrom
        self.mSbjctGenomeTo= this.mSbjctGenomeTo
        
        ## there might be some reference counting issues, thus
        ## do it the explicit way.
        alignlib.addAlignment2Alignment( this.mMapPeptide2Translation, other.mMapPeptide2Translation)
        self.mMapPeptide2Translation = alignlib.makeAlignmentVector()
        alignlib.addAlignment2Alignment( self.mMapPeptide2Translation, this.mMapPeptide2Translation )
        
        self.mTranslation = this.mTranslation
        
        self.mQueryFrom = self.mMapPeptide2Translation.getRowFrom()
        self.mQueryTo = self.mMapPeptide2Translation.getRowTo()
        self.mSbjctFrom = self.mMapPeptide2Translation.getColFrom()
        self.mSbjctTo = self.mMapPeptide2Translation.getColTo()
        
        self.mQueryCoverage = 100.0 * (self.mQueryTo - self.mQueryFrom + 1) / float(self.mQueryLength)

        self.mAlignmentString = string.join( map( \
                                      lambda x: string.join(map(str, x), " "),
                                      self.mMapPeptide2Genome), " ")

        f = alignlib.AlignmentFormatEmssions( self.mMapPeptide2Translation )
        self.mQueryAli, self.mSbjctAli = f.mRowAlignment, f.mColAlignment

        ## summary parameters
        self.mRank = max( this.mRank, other.mRank)
        self.score += other.score
        self.mNGaps += other.mNGaps
        self.mNFrameShifts += other.mNFrameShifts
        self.mNIntrons += other.mNIntrons + 1
        self.mNStopCodons += other.mNStopCodons
        
        nnew = self.mMapPeptide2Translation.getLength() - self.mMapPeptide2Translation.getNumGaps()
        
        self.mPercentIdentity = min( 100.0, (self.mPercentIdentity * nthis + other.mPercentIdentity * nother) / nnew )
        self.mPercentSimilarity = min( 100.0, (self.mPercentSimilarity * nthis + other.mPercentSimilarity * nother) / nnew )

        self.mNAssembled += 1 + other.mNAssembled

    def InvertGenomicCoordinates( self, lgenome ):
        """invert genomic alignment on sequence.

        Negative strand is calculated from the other end.
        """
        if self.mSbjctStrand == "-":
            x = self.mSbjctGenomeFrom 
            self.mSbjctGenomeFrom = lgenome - self.mSbjctGenomeTo
            self.mSbjctGenomeTo   = lgenome - x
        
    def shiftGenomicRegion( self, forward_offset, reverse_offset = 0 ):
        """shift aligned genomic region by offset.

        if reverse offset is given, negative strand predictions
        are shifted by this amount.
        """
        if reverse_offset:
            if self.mSbjctStrand == "+":
                self.mSbjctGenomeFrom += forward_offset
                self.mSbjctGenomeTo += forward_offset
            else:
                self.mSbjctGenomeFrom += reverse_offset                
                self.mSbjctGenomeTo += reverse_offset
        else:
            self.mSbjctGenomeFrom += forward_offset
            self.mSbjctGenomeTo += forward_offset

    def shiftQueryRegion( self, offset ):
        """shift aligned genomic region by offset."""
        self.mQueryFrom += offset
        self.mQueryTo += offset
        if self.mExpand:
            self.mMapPeptide2Translation.moveAlignment( offset, 0)

    def setStrand( self, strand ):
        """set strand."""
        self.mSbjctStrand = strand 

    def getHeader(self):
        """return header."""
        return "\t".join( ("id",
                          "query_token",
                          "sbjct_token",
                          "sbjct_strand",
                          "rank",
                          "score",
                          "query_from",
                          "query_to",
                          "query_ali",
                          "sbjct_from",
                          "sbjct_to",
                          "sbjct_ali",
                          "query_length",
                          "query_coverage",
                          "ngaps",
                          "nframeshifts",
                          "nintrons",
                          "nsplits",
                          "nstops",
                          "pidentity",
                          "psimilarity",
                          "translation",
                          "sbjct_genome_from",
                          "sbjct_genome_to",
                          "map_peptide2genome",
                          "nassembled" ) )
    def __str__( self ):
        """get a string representation of results."""

            
        if self.mExpand:
            if self.mMapPeptide2Translation.getLength() > 0:
                f = alignlib.AlignmentFormatEmissions( self.mMapPeptide2Translation )
                row_ali, col_ali = f.mRowAlignment, f.mColAlignment
                self.mQueryFrom = self.mMapPeptide2Translation.getRowFrom()
                self.mQueryTo = self.mMapPeptide2Translation.getRowTo()
                self.mSbjctFrom = self.mMapPeptide2Translation.getColFrom()
                self.mSbjctTo = self.mMapPeptide2Translation.getColTo()
            else:
                row_ali, col_ali = "", ""
        else:
            row_ali = self.mQueryAli
            col_ali = self.mSbjctAli
            
        if self.mPredictionId:
            return string.join( map(str, (\
                self.mPredictionId,
                self.mQueryToken,
                self.mSbjctToken,
                self.mSbjctStrand,
                self.mRank,
                self.score,
                self.mQueryFrom,
                self.mQueryTo,
                row_ali,
                self.mSbjctFrom,
                self.mSbjctTo,
                col_ali,
                self.mQueryLength,
                self.mQueryCoverage, 
                self.mNGaps,
                self.mNFrameShifts,
                self.mNIntrons,
                self.mNSplits,
                self.mNStopCodons,
                "%5.2f" % self.mPercentIdentity,
                "%5.2f" % self.mPercentSimilarity,
                self.mTranslation,
                self.mSbjctGenomeFrom,
                self.mSbjctGenomeTo,
                self.mAlignmentString,
                self.mNAssembled,
                )), "\t")
        else:
            return string.join( map(str, (\
              self.mQueryToken,
              self.mSbjctToken,
              self.mSbjctStrand,
              self.mRank,
              self.score,
              self.mQueryFrom,
              self.mQueryTo,
              row_ali,
              self.mSbjctFrom,
              self.mSbjctTo,
              col_ali,
              self.mQueryLength,
              self.mQueryCoverage, 
              self.mNGaps,
              self.mNFrameShifts,
              self.mNIntrons,
              self.mNSplits,
              self.mNStopCodons,
                "%5.2f" % self.mPercentIdentity,
                "%5.2f" % self.mPercentSimilarity,
              self.mTranslation,
              self.mSbjctGenomeFrom,
              self.mSbjctGenomeTo,
              self.mAlignmentString,
              self.mNAssembled,
              )), "\t")

    def fillFromTable( self, table_row ):

        if len(table_row) == 25:
            ( self.mPredictionId,
              self.mQueryToken, self.mSbjctToken, self.mSbjctStrand,
              self.mRank, self.score,
              self.mQueryFrom, self.mQueryTo, self.mQueryAli,
              self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
              self.mQueryLength, self.mQueryCoverage, 
              self.mNGaps, self.mNFrameShifts, self.mNIntrons,
              self.mNSplits, self.mNStopCodons,
              self.mPercentIdentity, self.mPercentSimilarity,
              self.mTranslation,
              self.mSbjctGenomeFrom, self.mSbjctGenomeTo,
              self.mAlignmentString) = table_row
        elif len(table_row) == 26:
            ( self.mPredictionId,
              self.mQueryToken, self.mSbjctToken, self.mSbjctStrand,
              self.mRank, self.score,
              self.mQueryFrom, self.mQueryTo, self.mQueryAli,
              self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
              self.mQueryLength, self.mQueryCoverage, 
              self.mNGaps, self.mNFrameShifts, self.mNIntrons,
              self.mNSplits, self.mNStopCodons,
              self.mPercentIdentity, self.mPercentSimilarity,
              self.mTranslation,
              self.mSbjctGenomeFrom, self.mSbjctGenomeTo,
              self.mAlignmentString,
              self.mNAssembled) = table_row[:26]
        elif len(table_row) > 26:
            ( self.mPredictionId,
              self.mQueryToken, self.mSbjctToken, self.mSbjctStrand,
              self.mRank, self.score,
              self.mQueryFrom, self.mQueryTo, self.mQueryAli,
              self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
              self.mQueryLength, self.mQueryCoverage, 
              self.mNGaps, self.mNFrameShifts, self.mNIntrons,
              self.mNSplits, self.mNStopCodons,
              self.mPercentIdentity, self.mPercentSimilarity,
              self.mTranslation,
              self.mSbjctGenomeFrom, self.mSbjctGenomeTo,
              self.mAlignmentString,
              self.mNAssembled) = table_row[:26]
        else:
            raise ValueError, "unknown format: %i fields" % len(data)
            sys.exit(0)
            
        if self.mExpand:
            self.mMapPeptide2Translation = alignlib.makeAlignmentVector()

            if self.mQueryAli != "" and self.mSbjctAli != "":
                alignlib.AlignmentFormatEmissions( self.mQueryFrom, self.mQueryAli,
                                                   self.mSbjctFrom, self.mSbjctAli ).copy( self.mMapPeptide2Translation )

            self.mMapPeptide2Genome = Genomics.String2Alignment( self.mAlignmentString )

    def setTranslation( self, genomic_sequence ):
        """set translation from genomic sequence."""
        self.mMapPeptide2Translation, self.mTranslation = Genomics.Alignment2PeptideAlignment( \
                self.mMapPeptide2Genome, self.mQueryFrom, self.mSbjctGenomeFrom, genomic_sequence )
        
    def getExonBoundariesOnGenome( self ):
        """return a list of exons for this prediction.
        """
        return map( lambda x: (x[3], x[4]),
                    Genomics.Alignment2ExonBoundaries( self.mMapPeptide2Genome,
                                                       self.mQueryFrom,
                                                       self.mSbjctGenomeFrom ))
        
    def write( self ):
        """write a tab separated list of results.
        """
        print str(self)
        
    def read( self, line ):

        data = string.split( line[:-1], "\t")
        
        if len(data) == 26:
            ( self.mPredictionId,
              self.mQueryToken, self.mSbjctToken, self.mSbjctStrand,
              self.mRank, self.score,
              self.mQueryFrom, self.mQueryTo, self.mQueryAli,
              self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
              self.mQueryLength, self.mQueryCoverage, 
              self.mNGaps, self.mNFrameShifts, self.mNIntrons,
              self.mNSplits, self.mNStopCodons,
              self.mPercentIdentity, self.mPercentSimilarity,
              self.mTranslation,
              self.mSbjctGenomeFrom, self.mSbjctGenomeTo,
              self.mAlignmentString, self.mNAssembled,
              ) = data
        elif len(data) == 25:
            ( self.mPredictionId,
              self.mQueryToken, self.mSbjctToken, self.mSbjctStrand,
              self.mRank, self.score,
              self.mQueryFrom, self.mQueryTo, self.mQueryAli,
              self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
              self.mQueryLength, self.mQueryCoverage, 
              self.mNGaps, self.mNFrameShifts, self.mNIntrons,
              self.mNSplits, self.mNStopCodons,
              self.mPercentIdentity, self.mPercentSimilarity,
              self.mTranslation,
              self.mSbjctGenomeFrom, self.mSbjctGenomeTo,
              self.mAlignmentString,
              ) = data
        elif len(data) == 24:            
            ( self.mQueryToken, self.mSbjctToken, self.mSbjctStrand,
              self.mRank, self.score,
              self.mQueryFrom, self.mQueryTo, self.mQueryAli,
              self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
              self.mQueryLength, self.mQueryCoverage, 
              self.mNGaps, self.mNFrameShifts, self.mNIntrons,
              self.mNSplits, self.mNStopCodons,
              self.mPercentIdentity, self.mPercentSimilarity,
              self.mTranslation,
              self.mSbjctGenomeFrom, self.mSbjctGenomeTo,
              self.mAlignmentString,
              ) = data
        elif len(data) == 23:
            ( self.mQueryToken, self.mSbjctToken, self.mSbjctStrand,
              self.mRank, self.score,
              self.mQueryFrom, self.mQueryTo, self.mQueryAli,
              self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
              self.mQueryLength, self.mQueryCoverage, 
              self.mNGaps, self.mNFrameShifts, self.mNIntrons,
              self.mNSplits, self.mNStopCodons,
              self.mPercentIdentity, self.mPercentSimilarity,
              self.mTranslation,
              self.mSbjctGenomeFrom, self.mSbjctGenomeTo,
              ) = data
            self.mAlignmentString = ""
        else:
            raise ValueError, "unknown format: %i fields in line %s" % (len(data), line[:-1])

        (self.score, self.mQueryCoverage, self.mPercentIdentity, self.mPercentSimilarity) = map (\
            float, (self.score, self.mQueryCoverage, self.mPercentIdentity, self.mPercentSimilarity))

        (self.mPredictionId, 
         self.mQueryFrom, self.mQueryTo, self.mQueryLength,
         self.mSbjctFrom, self.mSbjctTo,
         self.mSbjctGenomeFrom, self.mSbjctGenomeTo,
         self.mNGaps, self.mNIntrons, self.mNSplits, self.mNStopCodons,
         self.mNFrameShifts, self.mNAssembled) = map (\
            int, ( self.mPredictionId,
                   self.mQueryFrom, self.mQueryTo, self.mQueryLength,
                   self.mSbjctFrom, self.mSbjctTo,
                   self.mSbjctGenomeFrom, self.mSbjctGenomeTo,
                   self.mNGaps, self.mNIntrons, self.mNSplits, self.mNStopCodons,
                   self.mNFrameShifts, self.mNAssembled))

        if self.mExpand:        
            self.mMapPeptide2Translation = alignlib.makeAlignmentVector()

            if self.mQueryAli != "" and self.mSbjctAli != "":
                
                alignlib.AlignmentFormatExplicit(
                    self.mQueryFrom, self.mQueryAli,
                    self.mSbjctFrom, self.mSbjctAli).copy( self.mMapPeptide2Translation )

            self.mMapPeptide2Genome = Genomics.String2Alignment( self.mAlignmentString )

##--------------------------------------------------------------------------
class Predictions :
    mLogLevel = 0

    def __init__( self ):
        self.mMatches = []

    def __eq__(self, other):
        if not other: return 0        

        if not other.mMatches: return 0
        if not self.mMatches: return 0
        return self.mMatches[0] == other.mMatches[0]

    def __getitem__(self, key ):
        return self.mMatches[key]

    def __setitem__(self, key, value ):
        self.mMatches[key] = value

    def __len__(self):
        return len(self.mMatches)

    def __str__(self):
        return string.join( map(str, self.mMatches), "\n")

    def append( self, entry ):
        return self.mMatches.append( entry )
    
    def clear( self ):
        for x in self.mMatches:
            x.__del__()
        self.mMatches = []
    
    def combine(self, other):
        self.mMatches += other.mMatches

    def add( self, prediction ):
        """prediction to predictions."""
        self.mMatches.append(prediction)

    def reRank( self, offset = 1 ):
        """rerank predictions based on current sorting order."""
        
        for x in range(len(self.mMatches)):
            self.mMatches[x].mRank = offset + x

    def getBestMatch( self ):
        self.sort( lambda x,y: cmp(-x.score, -y.score)  )
        return self.mMatches[0]
        
    def getNumMatches( self ):
        """return number of matches."""
        return len(self.mMatches)

    def shiftGenomicRegion( self, forward_offset, reverse_offset = 0):
        """shift aligned genomic region by offset for all entries.

        if reverse offset is given, negative strand predictions
        are shifted by this amount.
        """
        for x in self.mMatches:
            x.shiftGenomicRegion( forward_offset, reverse_offset)

    def shiftQueryRegion( self, offset ):
        """shift aligned genomic region by offset for all entries."""
        for x in self.mMatches:
            x.shiftQueryRegion( offset )

    def setStrand( self, strand ):
        """set strand."""
        for x in self.mMatches:
            x.setStrand( strand )
    
    def write( self ):
        """write a tab separated list of results.
        """
        for x in self.mMatches:
            x.write()

    def sort( self, cmp_function = None ):
        """sort predictions.
        """
        self.mMatches.sort( cmp_function )

    def getRange( self ):
        """return a prediction covering all predictions."""

        p = self.mMatches[0].getCopy()

        p.mExpand = False
        
        if len(self.mMatches) == 1: return p
        
        for pp in self.mMatches[1:]:

            p.mQueryFrom       = min(p.mQueryFrom      , pp.mQueryFrom )
            p.mQueryTo         = max(p.mQueryTo        , pp.mQueryTo )
            p.mSbjctGenomeFrom = min(p.mSbjctGenomeFrom, pp.mSbjctGenomeFrom )
            p.mSbjctGenomeTo   = max(p.mSbjctGenomeTo  , pp.mSbjctGenomeTo )
            p.score += pp.score

        return p


def iterate( infile ):
    '''iterate over predicions in infile.'''

    for line in infile:
        if line.startswith("#"): continue
        if line.startswith("id\tquery_token"): continue
        p = Prediction()
        p.read( line )
        yield p
        

    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
graph_blast2pairs.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python graph_blast2pairs.py --help

Type::

   python graph_blast2pairs.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import tempfile
import time
import popen2

USAGE="""python %s [OPTIONS] < graph.in > graph.out

Version: $Id: graph_blast2pairs.py 2782 2009-09-10 11:40:29Z andreas $

Wrapper for rescoring blast alignments.

Options:
-h, --help                      print this message.
-v, --verbose=                  loglevel.
-m, --method=                   method to use for scoring [bitscore|normalize]
-f, --self-score=               filename with self-scores
--lambda=                       lambda for bitscore calculation
--k=                            K for bitscore calculation
--expected=                     expected score in matrix
-a, --append                    append new columns at the end
--evalue-to-log                 convert evalue to log
--effective-length=             calculate evalue from bitscore based on effective sbjct length
--min-evalue=                   minimum evalue (after which it is truncated)
--version                       ouptut version

new scores are saved in third column (overwriting the E-Value)

Methods:

kimura                          = set third column to kimura two parameter score from pid
bitscore                        = set third column to bitscore from score.
normalize-product               = normalize third column by dividing with self-scores
                                new = old * old / self1 / self2
normalize-max                   = normalize third column by max(old/self1), max(old/self2)
normalize-scoredist-avg         = normalize third column method by Sonnhammer & Hollich (2005)
scoredist-avg                   = set third column to score by Sonnhammer & Hollich (2005)
scoredist-min                   = set third column to modified score Sonnhammer & Hollich (2005) (minimum score for
                                        estimating the upper bound).
gapless-score                   = score of alignment without the gaps
reset-evalue                    = reset evalue based on bitscore
""" % sys.argv[0]

param_long_options = ["help", "verbose=", "method=", "lambda=", "k=", "self-scores=", 
                      "expected=", "append", "evalue-to-log",
                      "effective-length=", "version" ]

param_short_options = "hv:m:f:o:a"


param_loglevel = 1
param_method = "bitscore"
param_filename_self_scores = None

import CGAT.Experiment as E
import CGAT.BlastAlignments as BlastAlignments
import math

param_lambda = 0.267
param_K = 0.0410
param_expected = -0.5209
param_gop = -11.0
param_gep = -1.0

param_append = False
param_evalue_to_log = False

param_effective_length = None

param_min_evalue = 1e-200

def CalculateGapScore( ali, gop, gep ):

    s = 0.0
    data = re.split( "[+-]", ali[1:])
    for x in data[1:-1:2]:
        s += gop + int(x) * gep
    return s

##-------------------------------------------------------------------------------
if __name__ == "__main__":

    try:
        optlist, args = getopt.getopt(sys.argv[1:], param_short_options, param_long_options)
    except getopt.error, msg:
        print USAGE, msg
        sys.exit(2)

    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ( "-h", "--help" ):
            print USAGE
            sys.exit(0)
        elif o in ("-m", "--method"):
            param_method = a
        elif o in ("-f", "--self-scores"):
            param_filename_self_scores = a
        elif o == "--lambda":
            param_lambda = float(a)
        elif o == "--k":
            param_K = float(a)
        elif o == "--expected":
            param_expected = float(a)
        elif o in ("-a", "--append"):
            param_append = True
        elif o == "--evalue-to-log":
            param_evalue_to_log = True
        elif o == "--effective-length":
            param_effective_length = int(a)
        elif o == "--min-evalue":
            param_min_evalue = float(a)
            
    if param_loglevel >= 1:
        print E.GetHeader()
        print E.GetParams()

    if param_filename_self_scores:
        self_scores = {}
        for line in open(param_filename_self_scores, "r"):
            if line[0] == "#": continue
            d = line[:-1].split("\t")[:2]
            if d[0] not in self_scores: self_scores[d[0]] = 0.0
            self_scores[d[0]] = max( self_scores[d[0]], float(d[1]))

    if param_method == "kimura":
        a = 0
        f = lambda x: x < 0.85 and 0.0000001 - math.log( 1.0 - x - 0.2 * x * x ) or 5.2030

    elif param_method == "bitscore":
        a = 1        
        lK = math.log(param_K)
        l2 = math.log(2)
        f = lambda x: (param_lambda * x - lK) / l2
        
    elif param_method in ("normalize-product", "normalize-product-distance",
                          "normalize-max", "normalize-max-distance"):
        a = 2
        if param_method == "normalize-product":
            f = lambda x,y,z: x * x / self_scores[y] / self_scores[z]
        elif param_method == "normalize-product-distance":            
            f = lambda x,y,z: max( 0.0, 1.0 - x * x / self_scores[y] / self_scores[z])
        elif param_method == "normalize-max":
            f = lambda x,y,z: max( x / self_scores[y], x / self_scores[z])
        elif param_method == "normalize-max-distance":            
            f = lambda x,y,z: max( 0.0, 1.0 - max( x / self_scores[y], x / self_scores[z]))
            
    elif param_method == "normalize-scoredist-avg":
        a = 3
        f = lambda x,y,z,l: max(0.0, -100.0 * math.log( (x - param_expected * l) / ( (self_scores[y] + self_scores[z]) * 0.5 - param_expected * l ) ))

    elif param_method == "scoredist-avg":
        a = 4
        f = lambda x,y,z,l: max(0.0, -100.0 * math.log( (x - param_expected * l) / ( (self_scores[y] + self_scores[z]) * 0.5 - param_expected * l ) ))

    elif param_method == "scoredist-min":
        a = 4
        f = lambda x,y,z,l: max(0.0, -100.0 * math.log( (x - param_expected * l) / ( min(self_scores[y], self_scores[z]) - param_expected * l ) ))
    elif param_method == "gapless-score":
        a = 5
        f = lambda x,a,b: x - CalculateGapScore( a, param_gop, param_gep ) - CalculateGapScore( b, param_gop, param_gep)
    elif param_method == "reset-evalue":
        a = 6
        if param_evalue_to_log:
            # this way is less likely to underflow (2^-s might be zero for large s)
            me = math.log(param_min_evalue)
            l2 = math.log(2)
            le = math.log(param_effective_length)
            f = lambda s,m,n: max(me, -s * l2 + math.log(m) + le)
        else:
            f = lambda s,m,n: max(param_min_evalue, math.pow(2,-s) * m * n)
            
        param_evalue_to_log = False
    else:
        raise "unknown method %s" % param_method

    ninput, noutput, nskipped, nfailed = 0, 0, 0, 0

    for line in sys.stdin:

        if line[0] == "#": continue

        link = BlastAlignments.Link()
        link.Read( line )
        ninput += 1

        try:
            if a == 0:
                new_val = f( (100.0 - link.mPercentIdentity) / 100.0 )
            elif a == 1:
                new_val = f( link.score )
            elif a == 2:
                ## note: used to be evalue
                new_val = f( link.mBitScore, link.mQueryToken, link.mSbjctToken )
            elif a == 3:
                new_val = f( link.mEvalue, link.mQueryToken, link.mSbjctToken, max(link.mQueryTo - link.mQueryFrom, link.mSbjctTo - link.mSbjctFrom) + 1 )
            elif a == 4:
                new_val = f( link.score, link.mQueryToken, link.mSbjctToken, max(link.mQueryTo - link.mQueryFrom, link.mSbjctTo - link.mSbjctFrom) + 1 )
            elif a == 5:
                new_val = int(f( link.score, link.mQueryAli, link.mSbjctAli))
            elif a == 6:
                new_val = f( link.mBitScore, link.mQueryLength, param_effective_length)
                
        except KeyError:
            if param_loglevel >= 2:
                print "# Key error in line", line[:-1]
                nfailed += 1
            continue

        if param_evalue_to_log:
            link.mEvalue = math.log( link.mEvalue )
        
        if param_append:
            print str(link) + "\t" + str(new_val)
        else:
            if a in (0, 1, 2, 3, 4, 6):
                link.mEvalue = new_val
            elif a in (5,):
                link.score = new_val
            print str(link)
            
        noutput += 1
            
    print "# ninput=%i, noutput=%i, nskipped=%i, failed=%i" % (\
        ninput, noutput, nskipped, nfailed )

    if param_loglevel >= 1:    
        print E.GetFooter()





################################################################################
#   Gene prediction pipeline 
#
#   $Id: diff_gtf.py 2781 2009-09-10 11:33:14Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
diff_gtf.py - compare two genesets
===================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script compares two genesets in :term:`gtf`-formatted files.

It outputs the results of the comparison into various sections. The sections are
split into separate output files whose names are determined by the ``--output-filename-pattern`` 
option. The sections are:

genes_ovl
   Table with overlapping genes

genes_total
   Summary statistic of overlapping genes

genes_uniq1
   List of genes uniq in set 1

genes_uniq2
   List of genes uniq in set 2


Usage
-----

Example::

   python diff_gtf.py a.gtf b.gtf

Type::

   python diff_gtf.py --help

for command line help.

Command line options
--------------------

'''
import sys
import string
import re
import os
import optparse
import collections
import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
import bx.intervals.intersection

def GetNextLine( infile ):
    for line in infile:
        if line[0] == "#": continue
        return line
    return None

class Counts:

    mPercentFormat = "%.2f"

    def __init__(self, add_percent ):

        self.nleft, self.nright, self.noverlap = 0, 0, 0
        self.nunique_left = 0
        self.nunique_right = 0
        self.nidentical = 0
        self.nhalf = 0
        self.nsplit_left = 0
        self.nsplit_right = 0
        self.mAddPercent = add_percent
        
    def __add__(self, other):
        self.nleft += other.nleft
        self.nright += other.nright
        self.noverlap += other.noverlap
        self.nunique_left += other.nunique_left
        self.nunique_right += other.nunique_right
        self.nidentical += other.nidentical
        self.nhalf += other.nhalf
        self.nsplit_left += other.nsplit_left
        self.nsplit_right += other.nsplit_right
        return self
    
    def getHeader( self ):
        h = "total_left\ttotal_right\tnoverlap\tnidentical\tnhalf\tunique_left\tunique_right\tsplit_left\tsplit_right"        
        if self.mAddPercent:
            h += "\t" + self.getHeaderPercent()
        return h

    def __str__( self ):
        h = "\t".join(map(str, \
                             (self.nleft, self.nright,
                              self.noverlap, self.nidentical, self.nhalf,
                              self.nunique_left, self.nunique_right,
                              self.nsplit_left, self.nsplit_right)))
        if self.mAddPercent:
            h += "\t" + self.asPercent()
            
        return h 

    def getHeaderPercent( self ):
        return "\t".join( map( lambda x: "pl%s\tpr%s" % (x,x), ("overlap", "identical", "half", "unique", "split")))
    
    def asPercent( self ):
        return "\t".join( map( lambda x: self.mPercentFormat % (100.0 * x),
                               ( float(self.noverlap) / self.nleft,
                                 float(self.noverlap) / self.nright,    
                                 float(self.nidentical) / self.nleft,
                                 float(self.nidentical) / self.nright,    
                                 float(self.nhalf) / self.nleft,
                                 float(self.nhalf) / self.nright,    
                                 float(self.nunique_left) / self.nleft,
                                 float(self.nunique_right) / self.nright,    
                                 float(self.nsplit_left) / self.nleft,
                                 float(self.nsplit_right) / self.nright )))

def getFile( options, section ):

    if options.output_filename_pattern:
        outfile = IOTools.openFile(options.output_filename_pattern % section, "w")
        E.info( "output for section '%s' goes to file %s" % (section, options.output_filename_pattern % section) )
    else:
        outfile = options.stdout
        outfile.write( "## section: %s\n" % section)
    return outfile

def writeDiff( outfile, symbol, genes ):

    for gene in genes:
        for exon in gene:
            outfile.write("%s\t%s\n" % (symbol, str(exon)))

def main( argv = None ):

    if not argv: argv = sys.argv

    parser = E.OptionParser( version = "%prog version: $Id: diff_gtf.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-e", "--write-equivalent", dest="write_equivalent", 
                      help="write equivalent entries [default=%default].", action="store_true"  )

    parser.add_option("-f", "--write-full", dest="write_full",
                      help="write full gff entries [default=%default].", action="store_true"  )

    parser.add_option("-o", "--format=", dest="format",
                      help="output format [flat|multi-line] [default=%default]" )

    parser.add_option("-p", "--add-percent", dest="add_percent", action="store_true",
                      help="add percentage columns [default=%default]." )

    parser.add_option("-s", "--ignore-strand", dest="ignore_strand", action="store_true",
                      help="ignore strand information [default=%default]." )

    parser.set_defaults(
        write_equivalent = False,
        write_full = False,
        format="flat",
        add_percent=False,
        ignore_strand = False,
        as_gtf = False,
        )

    (options, args) = E.Start( parser, argv, add_output_options = True )

    if len(args) != 2:
        raise ValueError( "two arguments required" )

    input_filename1, input_filename2 = args

    ## duplicated features cause a problem. Make sure
    ## features are non-overlapping by running
    ## gff_combine.py on GFF files first.

    E.info( "reading data started" )

    idx, genes2 = {}, set()
    for e in GTF.readFromFile( IOTools.openFile( input_filename2, "r" ) ):
        genes2.add( e.gene_id )
        if e.contig not in idx: idx[e.contig] = bx.intervals.intersection.Intersecter()
        idx[e.contig].add_interval( bx.intervals.Interval(e.start,e.end,value=e) )

    overlaps_genes = []

    E.info( "reading data finished: %i contigs" % len(idx))

    subtotals = []
    subtotal = Counts( add_percent = options.add_percent )
    
    # outfile_diff and outfile_overlap not implemented
    # outfile_diff = getFile( options, "diff" )
    # outfile_overlap = getFile( options, "overlap" )
    overlapping_genes = set()

    genes1 = set()

    # iterate over exons
    with IOTools.openFile( input_filename1, "r" ) as infile:
        for this in GTF.iterator( infile ):

            genes1.add( this.gene_id )

            try:
                intervals = idx[this.contig].find( this.start, this.end )
            except KeyError:
                continue

            others = [ x.value for x in intervals ]
            for other in others:
                overlapping_genes.add( (this.gene_id, other.gene_id) )

            # check for identical/half-identical matches
            output = None
            for other in others:
                if this.start == other.start and this.end == other.end:
                    output, symbol = other, "="
                    break
            else:
                for other in others:
                    if this.start == other.start or this.end == other.end:
                        output, symbol = other, "|"
                        break
                else:
                    symbol = "~"

    # if outfile_diff != options.stdout: outfile_diff.close()
    # if outfile_overlap != options.stdout: outfile_overlap.close()

    outfile = None
    ##################################################################
    ##################################################################
    ##################################################################
    ## print gene based information
    ##################################################################    
    if overlapping_genes:
        outfile = getFile( options, "genes_ovl" )
        outfile.write ("gene_id1\tgene_id2\n" )
        for a, b in overlapping_genes: outfile.write( "%s\t%s\n" % (a,b) )
        if outfile != options.stdout: outfile.close()

        outfile_total = getFile(options, "genes_total" )
        outfile_total.write("set\tngenes\tnoverlapping\tpoverlapping\tnunique\tpunique\n" )

        outfile = getFile( options, "genes_uniq1" )
        b = set( [ x[0] for x in overlapping_genes ] )
        d = genes1.difference(b)
        outfile.write ("gene_id1\n" )
        outfile.write( "\n".join( d ) + "\n" )
        if outfile != options.stdout: outfile.close()
        outfile_total.write( "%s\t%i\t%i\t%5.2f\t%i\t%5.2f\n" % (
                os.path.basename(input_filename1), len(genes1), len(b), 100.0 * len(b) / len(a),
                len(d), 100.0 * len(d) / len(genes1) ) )

        outfile = getFile( options, "genes_uniq2" )
        b = set( [ x[1] for x in overlapping_genes ] )
        d = genes2.difference(b)
        outfile.write ("gene_id2\n" )
        outfile.write( "\n".join( d ) + "\n" )
        if outfile != options.stdout: outfile.close()

        outfile_total.write( "%s\t%i\t%i\t%5.2f\t%i\t%5.2f\n" % (
                os.path.basename(input_filename2), len(genes2), len(b), 100.0 * len(b) / len(a),
                len(d), 100.0 * len(d) / len(genes2) ) )
        if outfile_total != options.stdout: outfile_total.close()
        
    ##################################################################
    ##################################################################
    ##################################################################
    ## print totals
    ##################################################################    
    outfile = getFile(options, "total" )
    outfile.write("chr\tstrand\t%s\n" % Counts( add_percent = options.add_percent ).getHeader())

    total = Counts( add_percent = options.add_percent )
    for x in subtotals:
       outfile.write( "\t".join( (x[0], x[1], str(x[2])) ) + "\n" )
       total += x[2]

    outfile.write("\t".join( ("all", "all", str(total)) ) + "\n")
    if outfile != options.stdout: outfile.close()
    
    E.Stop()    

if __name__ == "__main__":
    sys.exit( main() )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
SetTools.py - Tools for working on sets
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Many of the functions in this module precede the :py:class:`set` datatype
in python.

Code
----

'''

import itertools
import numpy

def combinations( list_of_sets ):
    '''create all combinations of a list of sets

    returns a list of tuples ( set_composition, union, intersection )
    '''

    sr = range(len(list_of_sets))
    results = []
    for l in range(1, len(list_of_sets)):
        for combination in itertools.combinations( sr, l ):
            union = list_of_sets[combination[0]].union( *[list_of_sets[x] for x in combination[1:]])
            inter = list_of_sets[combination[0]].intersection( *[list_of_sets[x] for x in combination[1:]])

            results.append( (combination, union, inter ) )
    return results

def writeSets( outfile, list_of_sets, labels = None ):
    '''output a list of sets as a tab-separated file.

    *labels* is a list of set labels.
    '''
    
    all_ids = list_of_sets[0].union( *list_of_sets[1:] ) 
    if not labels: labels = range( len(list_of_sets))

    outfile.write("id\t%s\n" % "\t".join(map(str,labels)))

    for i in sorted(list(all_ids)):
        outfile.write( "%s\t%s\n" % (i,
                                  "\t".join( map(str, [ [0,1][i in x] for x in list_of_sets ] ) ) ))
    
def unionIntersectionMatrix( list_of_sets ):
    '''build union and intersection of a list of sets.

    return a matrix with the upper diagonal the union and the lower diagonal the intersection.
    '''

    l = len(list_of_sets)
    matrix = numpy.zeros( (l, l), dtype = numpy.int )
    for x in range( l ):
        xx = list_of_sets[x]
        for y in range( x ):
            yy = list_of_sets[y]
            union = xx.union( yy )
            inter = xx.intersection( yy )
            matrix[x][y] = len(union)
            matrix[y][x] = len(inter)

    return matrix

#------------------------------------------------------------------------
def CompareSets( set1, set2):
    """returns the union and the disjoint members of two sets. The sets have
    to be sorted.
    """

    unique1 = []
    unique2 = []
    common = []    

    x1 = 0
    x2 = 0
    while 1:

        if x1 >= len(set1) or x2 >= len(set2): break
        
        if set2[x2] == set1[x1]:
            common.append( set1[x1] )
            x1 += 1
            x2 += 1
            continue

        
        if set1[x1] < set2[x2]:
            unique1.append(set1[x1])
            x1 += 1
            continue
        
        if set1[x1] > set2[x2]:
            unique2.append( set2[x2] )
            x2 += 1
            
    if x2 < len(set2):
        unique2 += set2[x2:]
    elif x1 < len(set1):
        unique1 += set1[x1:]        
    
    return unique1, unique2, common
        
#------------------------------------------------------------------------
def MakeNonRedundant( set ):

    set.sort()

    last = -1
    result = []
    for x in set:
        if x != last:
            result.append(x)
        last = x
    
    return result

## The following is derived from the python recipe http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/410685
## by Zoran Isailovski 

def getAllCombinations( *sets ):

    if not sets: return []
    F = MakeListComprehensionFunction('F', len(sets) )
    return F(*sets)

def MakeListComprehensionFunction (name, nsets):
    """Returns a function applicable to exactly <nsets> sets.
    The returned function has the signature
    F(set0, set1, ..., set<nsets>)
    and returns a list of all element combinations as tuples.
    A set may be any iterable object.
    """
    if nsets <= 0:
        source = 'def %s(): return []\n' % name
    else:
        constructs = [ ('set%d'%i, 'e%d'%i, 'for e%d in set%d'%(i,i))
                       for i in range(nsets) ]
        a, e, f = map(None, *constructs)
        ##e.reverse() # <- reverse ordering of tuple elements if needed
        source = 'def %s%s:\n   return [%s %s]\n' % \
                 (name, _tuplestr(a), _tuplestr(e), ' '.join(f))
    scope = {}
    exec source in scope
    return scope[name]
    
def _tuplestr(t):
    if not t: return '()'
    return '(' + ','.join(t) + ',)'

def xuniqueCombinations(items, n):
    if n==0: yield []
    else:
        for i in xrange(len(items)):
            for cc in xuniqueCombinations(items[i+1:],n-1):
                yield [items[i]]+cc





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
data2stats.py - summary statistics on table columns/rows
========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script computes summary statistics (min, mean, median)
on rows/columns of a table.

Usage
-----

Example::

   python data2stats.py < table.in > stats.out

Type::

   python data2stats.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import optparse

import CGAT.Experiment as E
import CGAT.Histogram as Histogram
import scipy

def PrintValues( outfile, values,  options, prefix = "",titles = None):

    if options.flat or options.aggregate_column:

        if options.add_header:
            if prefix: outfile.write( "prefix\t" )
            
            if titles: outfile.write( "column\t" )
                
            print "\t".join( ("nval", "min", "max", "mean", "median", "stddev", "sum", "q1", "q3" ) )
        
        for x in range(len(values)):

            vals = values[x]

            if len(vals) == 0:

                if options.output_empty:
                    if titles: outfile.write( titles[x] + "\t" )
                    if prefix: outfile.write( prefix + "\t" )

                    outfile.write( "0" + "\tna" * 8  + "\n" )

                continue

            if titles: outfile.write( titles[x] + "\t" )
            if prefix: outfile.write( prefix + "\t" )

            vals.sort()
            if len(vals) > 4:
                q1 = options.value_format % vals[len(vals) // 4]
                q3 = options.value_format % vals[len(vals) * 3 // 4]
            else:
                q1 = options.value_format % vals[0]
                q3 = options.value_format % vals[-1]

            outfile.write( "\t".join( ( "%i" % len(vals),
                                        options.value_format % float(min(vals)),
                                        options.value_format % float(max(vals)),
                                        options.value_format % scipy.mean(vals),
                                        options.value_format % scipy.median(vals),
                                        options.value_format % scipy.std(vals),                                      
                                        options.value_format % reduce( lambda x, y: x+y, vals),
                                        q1, q3,
                                        )) + "\n")
            
    else:

        if titles:
            print "category\t%s" % string.join(titles,"\t")

        print "count\t%s"  % (string.join( map(lambda v: "%i" % len(v), values), "\t"))
        print "min\t%s"    % (string.join( map(lambda v: options.value_format % min(v), values), "\t"))
        print "max\t%s"    % (string.join( map(lambda v: options.value_format % max(v), values), "\t"))
        print "mean\t%s"   % (string.join( map(lambda v: options.value_format % scipy.mean(v), values), "\t"))
        print "median\t%s" % (string.join( map(lambda v: options.value_format % scipy.median(v), values), "\t"))
        print "stddev\t%s" % (string.join( map(lambda v: options.value_format % scipy.std(v), values), "\t"))
        print "sum\t%s"    % (string.join( map(lambda v: options.value_format % reduce( lambda x,y: x+y, v), values), "\t"))
        print "q1\t%s"     % (string.join( map(lambda v: options.value_format % scipy.stats.scoreatpercentile(v,per=25), values), "\t"))
        print "q3\t%s"     % (string.join( map(lambda v: options.value_format % scipy.stats.scoreatpercentile(v,per=75), values), "\t"))

##---------------------------------------------------------------------------------------------------------        
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: data2stats.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to take for calculating histograms." )
    parser.add_option("--min-value", dest="min_value", type="float",
                      help="minimum value for histogram.")
    parser.add_option("--max-value", dest="max_value", type="float",
                      help="maximum value for histogram.")
    parser.add_option("--scale", dest="scale", type="float",
                      help="scale values.")
    parser.add_option("-a", "--aggregate-column", dest="aggregate_column", type="int",
                      help="use column to aggregate."  )
    parser.add_option("-i", "--no-title", dest ="titles", action="store_false",
                      help="do not use supplied column titles." )
    parser.add_option("-e", "--headers", dest ="headers", type="string",
                      help="headers." )
    parser.add_option("-r", "--rows", dest="rows", action="store_true",
                      help="data is in rows."  )
    parser.add_option("--ignore-zeros", dest="ignore_zeros", action="store_true",
                      help="ignore zero values."  )
    parser.add_option("-f", "--format", dest="value_format", type="string",
                      help="number format."  )
    parser.add_option("-x", "--flat", dest="flat", action="store_true",
                      help="flat format."  )
    parser.add_option("--skip-header", dest="add_header", action="store_false",
                      help="do not add header to flat format."  )
    parser.add_option("--write-header", dest="write_header", action="store_true",
                      help="write header and exit."  )
    parser.add_option("--skip-empty", dest="output_empty", action="store_false",
                      help="do not output empty columns."  )
    parser.add_option("--output-empty", dest="output_empty", action="store_true",
                      help="output empty columns."  )

    parser.set_defaults(
        columns = "all",
        min_value = None,
        max_value = None,
        scale = None,
        aggregate_column = None,
        titles = True,
        headers = None,
        rows = False,
        value_format = "%6.4f",
        flat = False,
        test = "%5.2f",
        add_header = True,
        write_header = False,
        ignore_zeros = False,
        output_empty = False,
        separator = "\t"
        )
    


    (options, args) = E.Start( parser, quiet = True )

    if options.columns not in ( "all", "all-but-first", "variable" ):
        options.columns = map(lambda x: int(x) -1 , options.columns.split(","))

    if options.headers: options.headers = options.headers.split(",")
    
    ## write header for flat output
    if options.write_header:
        print "\t".join( ("nval", "min", "max", "mean", "median", "stddev", "sum", "q1", "q3") )
        sys.exit(0)

    # retrieve histogram
    lines = filter( lambda x: x[0] <> "#", sys.stdin.readlines())

    outfile = sys.stdout

    if len(lines) > 0:
        
        ncols = len(string.split(lines[0][:-1], "\t"))
        
        if options.columns == "all":
            options.columns = range(0,ncols)
        elif options.columns == "all-but-first":
            options.columns = range(1,ncols)
        elif options.columns == "variable":
            pass
            
        if options.rows:

            ## ignore first value: is row title            
            if options.columns != "variable":
                del options.columns[0]

            if options.titles: del lines[0]
                
            ## write header for flat output
            if options.flat:
                if options.headers:
                    x = options.headers[0]
                else:
                    x = "row"
                print "\t".join( (x, "nval", "min", "max", "mean", "median", "stddev", "sum", "q1", "q3") )
                options.add_header = False
                
            for l in lines:
                data = l[:-1].split(options.separator)

                if options.columns == "variable":
                    vals = data[1:]
                else:
                    vals = map(lambda x: data[x], options.columns)

                ## remove unknown values
                vals= [ float(x) for x in vals if x and x.lower() not in ("na", "nan" ) ]

                if options.ignore_zeros:
                    vals = [ x for x in vals if x != 0.0 ]

                ## now convert to float
                vals = map(float, vals)
                
                PrintValues( outfile, [vals], options, data[0] )
                
        else:

            last_aggregate = None

            if options.titles:
                data = lines[0][:-1].split("\t")
                
                if not options.headers:
                    options.headers = map( lambda x: data[x], options.columns )
                    del lines[0]

                if options.aggregate_column != None:
                    print "category\t%s" % string.join(options.headers,"\t")
                
            vals = [ [] for x in range(len(options.columns)) ]

            for l in lines:

                data = l[:-1].split("\t")

                for c in range(len(options.columns)):

                    try:
                        val = string.atof(data[options.columns[c]])

                    except IndexError:
                        print "# IndexError in line:", l[:-1]
                        continue
                    except ValueError:
                        continue

                    if options.aggregate_column != None:

                        if last_aggregate != data[options.aggregate_column]:

                            if last_aggregate:
                                PrintValues( outfile, vals, options, last_aggregate )

                            vals = [ [] for x in range(len(options.columns)) ]
                            last_aggregate = data[options.aggregate_column]

                    if options.scale:
                        val *= options.scale

                    if options.max_value != None and val > options.max_value:
                        val = options.max_value

                    if options.min_value != None and val < options.min_value:
                        val = options.min_value

                    vals[c].append( val )

            lines = None

            ## remove empty columns
            nvals = []
            titles = []
            for c in range(len(options.columns)):
                if vals[c] or options.output_empty:
                    nvals.append( vals[c] )

                    if options.headers:
                        titles.append( options.headers[c] )

            vals = nvals
            
            PrintValues( outfile, vals, options, last_aggregate, titles )
    
    else:
        if options.titles:
            titles = ["missing",]
        else:
            titles = []

        if options.output_empty:
            PrintValues( outfile, [ [], ], options, None, titles )

    if options.loglevel >= 1:
        E.Stop()
    













################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
graph_links2gdl.py - output graph in gdl format
===============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a :term:`graph` in :term:`edge list` format and
converts it into :term:`gdl` format. The :term:`gdl` formatted
graph can then be layouted and visualized with :file:`aisee`.

Usage
-----

Example::

   python graph_links2gdl.py --help

Type::

   python graph_links2gdl.py --help

for command line help.

Documentation
-------------

Code
----

'''
USAGE = """
Create a graph from a list of edges.

python links2gdl.py [OPTIONS] < in_edges > out_graph.gdl

--components_file=      file of components in format token\tcomponent_id
--component=            only output component component_id
--subset=               file of tokens that graph is restricted to
--bipartite             consider file to be a bipartite graph (left| right) are different sets of vertices
--no_titles             input graph has no column headers. Default is to ignore the first line.
--weights               add weights to graph
--default-edge-colour   default edge colour
"""

import os
import sys
import string
import re
import getopt
import optparse

import CGAT.Experiment as E
import matplotlib

FORMAT_GRAPH = '''
         layoutalgorithm: forcedir      
         scaling        : maxspect
         arrowmode      : free
         
         attraction     : 60    // Attractive force           
         repulsion      : 60    // Repulsive force            
         gravity        :  0.0  // Gravitational force (float)
         randomrounds   :  0    // Nr.rounds with rand.impulse
         randomimpulse  :  0    // Force of the random impulse
         fdmax          : 50    // Number Iterations          
         tempmax        : 254   // Maximal Temperature        
         tempmin        :  0    // Minimal Temperature        
         temptreshold   :  3    // > 0                        
         tempscheme     :  3    // 1 - 8                      
         tempfactor     :  1.08 // > 1 (float)                
         randomfactor   : 100   // 100 means: determ. schedule
         magnetic_field1: top_to_bottom
         magnetic_field2: no
         magnetic_force1: 20
         magnetic_force2:  0
         border x : 5000
         border y : 5000
'''

FORMAT_EDGE = '''
         edge.arrowstyle: none
         edge.thickness: 1
'''
         
FORMAT_NODE = """
        node.shape      : circle
        node.height     : 32
        node.width      : 32
        node.color      :  lightgrey
        node.fontname   : "timR08.vcf"
        node.textcolor  :  0
        node.bordercolor:  black
"""

FORMAT_NODE_BIPARTITE = """
        node.shape      : box
        node.height     : 32
        node.width      : 32
        node.color      :  red
        node.fontname   : "timR08.vcf"
        node.textcolor  :  0
        node.bordercolor:  black
"""

def PrintNodes( nodes, labels, colours):
    
    for id in nodes.keys():

        if labels.has_key(id):
            label = labels[id]
        else:
            label = "na"

        if colours.has_key(id):
            colour = colours[id]
        else:
            colour = None
            
        if colour:
            print '\tnode: { label: "%s" title: "%s" info1: "%s" info2: "%s" color: %s}' % (id,id,label,id,colour)
        else:
            print '\tnode: { label: "%s" title: "%s" info1: "%s" info2: "%s" }' % (id,id,label,id)            
            
    

##---------------------------------------------------------------------------------------------------------
if __name__ == "__main__":

    #--------------------------------------------------------
    # command line parsing options
    parser = E.OptionParser( version = "%prog version: $Id: graph_links2gdl.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-c", "--filename-components", dest="filename_components", type="string",
                      help="filename with component information." )
    parser.add_option("-o", "--component", dest="component", type="string",
                      help="restrict output to component." )
    parser.add_option("-s", "--filename-node-colours", dest="filename_node_colours", type="string",
                      help="filename with node colours." )
    parser.add_option("-l", "--filename-node-labels", dest="filename_node_labels", type="string",
                      help="filename with node labels." )
    parser.add_option("-w", "--weights", dest="weights", action="store_true",
                      help="use weights." )
    parser.add_option( "--edge-colour", dest="edge_colour", type="choice",
                       choices=("heat", "jet", ),
                       help="colour edges." )
    parser.add_option( "--format-node", dest="filename_node_format", type="string",
                       help="filename with node formats." )
    parser.add_option( "--format-edge", dest="filename_edge_format", type="string",
                       help="filename with edge formats." )
    parser.add_option( "--format-graph", dest="filename_graph_format", type="string",
                       help="filename with graph format." )
    parser.add_option( "--no-titles", dest="titles", action="store_false",
                       help="do not output titles." )
    parser.add_option( "--weight-range", dest="weight_range", type="string",
                       help="weight range. Comma separated values or 'auto'")

    parser.add_option( "--add-edge-labels", dest="add_edge_labels", action="store_true",
                       help="add edge labels to edges." )
                       
    parser.set_defaults(
        min_weight = 0.0,
        max_weight = 1.0,
        num_colours = 100,
        min_colour = 16,
        filename_components = None,
        component = None,
        filename_subset = None,
        colours = None,
        filename_labels  = None,
        weights = False,
        edge_colour = None,
        default_edge_colour = "18",
        filename_format_graph = None,
        filename_format_edge = None,
        filename_format_node = None,
        filename_format_bipartite = None,
        titles = True,
        edge_labels = True,
        column_edge_weight = 3,
        column_edge_colour = 3,
        weight_range = "auto",
        add_edge_labels = False,
        )

    (options, args) = E.Start( parser )

    options.column_edge_weight -= 1
    options.column_edge_colour -= 1    

    components = None
    ## read components
    if options.filename_components:
        lines = open(options.filename_components,"r").readlines()
        components = {}
        for line in lines:
            id, cid = string.split(line[:-1], "\t")
            components[id] = cid

    if options.filename_subset:
        lines = open(options.filename_subset,"r").readlines()        
        subset= {}
        for line in lines:
            id = string.split(line[:-1], "\t")[0]
            subset[id] = 1
        
    colours = {}            
    if options.filename_node_colours:
        infile = open(options.filename_node_colours, "r")
        while 1:
            line = infile.readline()
            if not line: break
            if line[0] == "#": continue
            id, colour = string.split(line[:-1], "\t")[:2]
            colours[id] = colour
        infile.close()
    
    labels = {}
    if options.filename_labels:
        infile = open(options.filename_labels, "r")
        while 1:
            line = infile.readline()
            if not line: break
            if line[0] == "#": continue            
            id, label = string.split(line[:-1], "\t")[:2]
            if labels.has_key(id):
                labels[id] += "," + label
            else:
                labels[id] = label
                
        infile.close()

    if options.weight_range == "auto":
        lines = sys.stdin.readlines()
        mi, ma = None, None

        for line in lines:

            if line[0] == "#": continue
            try:
                v = float(line[:-1].split("\t")[2])
            except ValueError:
                continue
            except IndexError:
                continue
            
            if mi == None: 
                mi = v
            else:
                mi = min(v, mi)
            if ma == None:
                ma = v
            else:
                ma = max(v, ma)

        options.min_weight = mi
        options.max_weight = ma
        if options.loglevel >= 1:
            options.stdlog.write("# using automatic weight range from %f to %f\n" % (options.min_weight, options.max_weight) )
    else:
        lines = sys.stdin
        options.min_weight, options.max_weight = map(float, options.weight_range.split(","))

    options.stdout.write( "graph: {\n" )
    
    if options.filename_format_graph:
        options.stdout.write( string.join(open(options.filename_format_graph, "r").readlines() ) )
    else:
        options.stdout.write( FORMAT_GRAPH )

        if options.add_edge_labels:
            options.stdout.write( "display_edge_labels: yes\n" )
        
    left_nodes = {}
    right_nodes = {}
    touched = {}        # remove repeats (several links between the same nids)

    if options.edge_colour :
        import matplotlib
        import pylab

        colour_map = pylab.get_cmap( options.edge_colour )
        
        if options.edge_colour == "jet":
            
            # for some reason I had problems with aisee colour indices > 128. Thus,
            # I suggest staying below 128.
            step_size = (options.max_weight - options.min_weight) / float(options.num_colours+1)
            increment = 1.0 / float(options.num_colours+1)
            v, vv = options.min_weight, 0.0
            for x in range( options.min_colour, options.min_colour + options.num_colours + 1):
                rgba = colour_map( vv )
                r, g, b = map( lambda x: int(x * 255.0), rgba[:3] )
                options.stdout.write( "         colorentry %i: %03i %03i %03i // weight = %f\n" %\
                                          (x,r,g,b,v ) )
                v += step_size
                vv += increment

            colour_scale = options.num_colours / (options.max_weight - options.min_weight ) 
            def calc_colour_index( x ):
                v = min( options.max_weight, float(x))
                v = max( options.min_weight, v ) - options.min_weight
                v = min( options.num_colours - 1, int( v * colour_scale )) + options.min_colour
                return str( v )

            colour_conversion = calc_colour_index

            # save legend
            a=pylab.outerproduct(pylab.arange(options.min_weight,options.max_weight,step_size),1)
            pylab.figure(figsize=(10,5))
            pylab.imshow(a,cmap=colour_map,origin="lower")
            pylab.colorbar()
            pylab.savefig("legend.png",dpi=100,facecolor='gray')
        else:
            raise "unknown colour scheme", options.colour_scheme


    if options.filename_format_edge:
        options.stdout.write( string.join(open(options.filename_format_edge, "r").readlines()) )
    else:
        options.stdout.write( FORMAT_EDGE )

    first = True
    for line in lines:

        if line[0] == "#": continue

        if first:
            first = False
            if options.titles: continue

        try:
            weight = 1.0
            colour = options.default_edge_colour
            
            x = string.split(line[:-1], "\t")

            id1, id2 = x[:2]
            
            if options.edge_colour:
                colour = colour_conversion(x[options.column_edge_colour])
            if options.edge_labels:
                weight = x[options.column_edge_weight]

        except ValueError:
            continue

        ## patch for PFAM domains
        ## id1, id2 = map(string.atoi, string.split(line[:-1], "\t")[:2])

        if id1 == id2: continue
        
        if options.filename_subset:
            if not subset.has_key(id1) or not subset.has_key(id2):
                continue

        if options.component != None:
            if not components.has_key(id1):
                continue
            if not components.has_key(id2):
                continue
            if components[id1] != options.component or components[id2] != options.component:
                continue

        if options.filename_format_bipartite:
            left_nodes[id1] = 1
            right_nodes[id2] = 1
        else:
            left_nodes[id1] = 1
            left_nodes[id2] = 1
            
        if id1 < id2:
            key = "%s-%s" % (id1,id2)
        else:
            key = "%s-%s" % (id2,id1)          
  
        if touched.has_key(key):
            continue
        touched[key] = 1
            
        edge_attributes = [ 'color: %s' % colour ]
        if options.add_edge_labels:
            edge_attributes.append( 'label: "%s"' % weight )

        options.stdout.write('\tedge: { thickness: 3 sourcename: "%s" targetname: "%s" %s}\n' % (id1, id2, " ".join(edge_attributes)) )

    # sort nodes according to key

    if options.filename_format_node:
        options.stdout.write( "".join(open(options.filename_format_node, "r").readlines()))
    else:
        options.stdout.write( FORMAT_NODE )

    PrintNodes( left_nodes, labels, colours)

    if options.filename_format_bipartite:
        if options.format_bipartite == "default":
            options.stdout.write( FORMAT_BIPARTITE )
        else:
            options.stdout.write( "".join(open(options.filename_format_bipartite, "r").readlines()))
            
        PrintNodes( right_nodes, labels, colours)
            
    options.stdout.write ("}\n" )
    
    E.Stop()

    











"""
Glam2Scan.py - Parser for MAST files
====================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

As of biopython 1.5.6, the MAST parser is broken.

"""

import string, sys, re, types

class Glam2Scan:
    # number of matches
    def __init__(self): 
        self.motifs = []
        self.matches = []

class Motif: pass

class Match:
    """a Glam2Scan entry.
    """
    header = "\t".join( ("id", "start", "end", "strand", "score") )

    def __init__(self):
        pass

    def __str__(self):
        return "\t".join( map(str,
                              (self.id,
                               self.start,
                               self.end,
                               self.strand,
                               self.sequence,
                               self.score)))

def parse( infile ):
    '''parse Glam2Scan output.'''

    x = -1
    for line in infile:
        if line.startswith( "GLAM2SCAN"): x = 4
        x -= 1
        if x == 0: break

    if not line:
        raise ValueError("could not find tag 'GLAM2SCAN'")

    gl = Glam2Scan()
    keep = False
    for line in infile:
        if line.strip() == "": continue
        if line.startswith(" "): continue
        m = Match()
        d = re.split("\s+", line[:-1])
        if len(d) != 6: break
        m.id, m.start, m.sequence, m.end, m.strand, m.score =\
            d[0], int(d[1])-1, d[2], int(d[3]), d[4], float(d[5])
        gl.matches.append( m )

    return gl




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperGblocks.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile

"""Wrapper for Gblocks
"""

import Genomics

class Gblocks:

    mOptions = ""
    mExecutable = "Gblocks %s -t d"
    mEnvironment = ""
    
    def __init__( self, options = ""):
        self.mOptions = options

    def GetBlocks( self, s1, s2 ):
        """the strings have to be already aligned!!!"""
        
        handle_tmpfile, filename_tmpfile = tempfile.mkstemp()
        os.write( handle_tmpfile, ">s1\n%s\n" % (s1))
        os.write( handle_tmpfile, ">s2\n%s\n" % (s2))
        os.close( handle_tmpfile )

        statement = " ".join( "(", self.mEnvironment, 
                              self.mExecutable % filename_tmpfile,
                              self.mOptions, ")" )

        p = subprocess.Popen( statement , 
                              shell=True, 
                              stdin=subprocess.PIPE, 
                              stdout=subprocess.PIPE, 
                              stderr=subprocess.PIPE, 
                              close_fds=True)

        (file_stdout, file_stdin, file_stderr) = (p.stdin, p.stdout, p.stderr)

        file_stdin.close()
        lines = file_stdout.readlines()
        lines_stderr = file_stderr.readlines()
        exit_code = file_stdout.close()
        file_stderr.close()
        if exit_code:
            raise ValueError("Error while executing statement %s" % statement)

        if not os.path.exists( filename_tmpfile + "-gb"):
            os.remove( filename_tmpfile )            
            return "", ""
        
        lines = open( filename_tmpfile + "-gb").readlines()
        r = Genomics.ParseFasta2Hash( lines)

        if not r: return "", ""

        os.remove( filename_tmpfile )
        os.remove( filename_tmpfile + "-gb" )
        os.remove( filename_tmpfile + "-gb.htm")        
        
        return r['s1'], r['s2']




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
ProgressBar.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import sys, os, string, time

class ProgressBar:
    """progress bar class

    adapted from Randy Pargman (2002)
    see http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/168639
    """
    
    def __init__(self, minValue = 0, maxValue = 10, totalWidth=12):
        
        self.progBar = "[]"   # This holds the progress bar string
        self._old_pbar = ""
        self.min = minValue
        self.max = maxValue
        self.span = maxValue - minValue
        self.width = totalWidth
        self.amount = 0       # When amount == max, we are 100% done
        self.updateAmount(0)  # Build progress bar string

    def updateAmount(self, newAmount = 0):

        if newAmount < self.min: newAmount = self.min
        if newAmount > self.max: newAmount = self.max
        self.amount = newAmount

        # Figure out the new percent done, round to an integer
        diffFromMin = float(self.amount - self.min)
        percentDone = (diffFromMin / float(self.span)) * 100.0
        percentDone = round(percentDone)
        percentDone = int(percentDone)

        # Figure out how many hash bars the percentage should be
        allFull = self.width - 2
        numHashes = (percentDone / 100.0) * allFull
        numHashes = int(round(numHashes))

        # build a progress bar with hashes and spaces
        self.progBar = "[" + '#'*numHashes + ' '*(allFull-numHashes) + "]"

        # figure out where to put the percentage, roughly centered
        percentPlace = (len(self.progBar) / 2) - len(str(percentDone))
        percentString = str(percentDone) + "%"

        # slice the percentage into the bar
        self.progBar = self.progBar[0:percentPlace] + percentString + \
                       self.progBar[percentPlace+len(percentString):]

    def draw(self):
        # draw progress bar - but only if it has changed

        if self.progBar != self._old_pbar:
            self._old_pbar = self.progBar
            sys.stdout.write(self.progBar + '\n')
            sys.stdout.flush()

    def __str__(self):
        return str(self.progBar)


if __name__ == "__main__":

    mi, ma= 0, 100
    bar = ProgressBar( 0, 100, 20 )

    for x in range(mi, ma):
        time.sleep(1)
        bar.updateAmount( x )
        bar.draw()
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
quality2fasta.py - convert quality scores to fasta/fastq output
===============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Convert quality scores to a fasta or fastq formatted file. 
Quality scores are positive and mapped to the range [a-zA-Z0-9] 
starting at 0.

Usage
-----

Example::

   python quality2fasta.py --help

Type::

   python quality2fasta.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import getopt
import time
import optparse

import CGAT.Experiment as E
import CGAT.IndexedFasta as IndexedFasta
import CGAT.FastaIterator as FastaIterator

class FromFastaIterator:

    def __init__(self, infile, alphabet = "fastq", encoding = "phred", default = None ):
        """default: set all quality scores to this default value."""

        import FastaIterator
        self.mInputIterator = FastaIterator.FastaIterator( infile )
        self.mOutputIterator = self._iterate()
        self.mNOverFlow = 0
        self.mNUnderFlow = 0
        self.mDefault = default

        # how to convert a phred like score to a character:
        if alphabet == "fastq":
            self.mMapScore2Char = [ chr(33 + x) for x in range( 0, 93) ]
        elif alphabet == "solexa":
            self.mMapScore2Char = [ chr(64 + x) for x in range( 0, 128) ]
        elif alphabet == "printable":
            self.mMapScore2Char = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
        else:
            raise "unknown alphabet %s" % alphabet

        if encoding == "phred":
            # no change for phred scores
            self.mMapScore2Score = range( 0, 93 )
        elif encoding == "solexa":
            # solexa encoding
            self.mMapScore2Score =[ int(10.0 * log( 1.0 + 10 ** (x / 10.0)) / log(10)+.499) for x in range(-64,65) ]

        self.mOverFlow = "^"
        self.mUnderFlow = "."
        self.mNInput = 0
        self.mNOutput = 0

    def __iter__(self):
        return self

    def next(self):
        return self.mOutputIterator.next( )

    def _iterate( self ):
        """iterate over muliple files."""
        
        while 1:
            cur_entry = self.mInputIterator.next()

            self.mNInput += 1
            if self.mDefault:
                values = [ self.mDefault ] * len( re.sub( "\s", "", cur_entry.sequence ) )
            else:
                values = map( int, re.split( " +", cur_entry.sequence.strip() ) )

            s = []
            for v in values:
                
                v = self.mMapScore2Score[v]

                if v < 0:
                    c = self.mUnderflow
                    self.mNUnderFlow += 1
                try:
                    c = self.mMapScore2Char[v]
                except IndexError:
                    c = self.mOverFlow
                    self.mNOverFlows += 1

                s.append( c )
            
            self.mNOutput += 1
            yield cur_entry.title.strip(), "".join(s)

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: quality2fasta.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option("-f", "--format", dest="format", type="choice",
                      choices=("fasta", ),
                      help="input format [%default]."  )

    parser.add_option("-o", "--output-format", dest="output_format", type="choice",
                      choices=("fasta", "fastq" ),
                      help="output format - if fastq is chosen, also supply a sequence file [%default]."  )
    
    parser.add_option("-a", "--alphabet", dest="alphabet", type="choice",
                      choices=("fastq", "solexa", "printable" ),
                      help="characters to use for quality scores [%default]."  )

    parser.add_option("-e", "--encoding", dest="encoding", type="choice",
                      choices=("phred", "solexa" ),
                      help="encoding of quality scores [%default]."  )
    
    parser.add_option("-i", "--build-index", dest="build_index", type="string",
                      help="build an index. Supply the database name [%default]."  )

    parser.add_option("-s", "--filename-sequences", dest="filename_sequences", type="string",
                      help="input filename with file of sequences in fasta format - sorted in the same way as the quality file [%default]."  )


    parser.add_option( "-d", "--set-to-default", dest="default_value", type="int",
                       help="set all quality codes to the default value. Supply the fasta sequence instead of the quality codes [%default]." )

    parser.set_defaults(
        format = "fasta",
        output_format = "fasta",
        build_index = None,
        filename_sequences = None,
        alphabet = "fastq",
        encoding = "phred",
        default_value = None,
        )
    
    (options, args) = E.Start( parser )

    ninput, noutput = 0, 0
    
    if options.format == "fasta":
        iterator = FromFastaIterator( sys.stdin, alphabet = options.alphabet, default = options.default_value )

    if options.output_format == "fasta":

        if options.build_index:
            IndexedFasta.createDatabase( options.build_index,
                                         iterator )
        else:
            while 1:
                try:
                    r = iterator.next()
                except StopIteration:
                    break
                t,s = r
                options.stdout.write( ">%s\n%s\n" % (t,s))

    elif options.output_format == "fastq":
        
        if not options.filename_sequences:
            raise "please supply a filename with sequences."

        iterator_sequence = FastaIterator.FastaIterator( open( options.filename_sequences, "r" ) )
        
        while 1:
            qual, seq = None, None
            try:
                qual = iterator.next()
                seq = iterator_sequence.next()
            except StopIteration:
                if qual and not seq:
                    options.stdlog.write( "# sequence file incomplete\n" )
                elif seq and not qual:
                    options.stdlog.write( "# quality file incomplete\n" )

            qt, qs = qual
            st, ss = seq.title, seq.sequence
            assert qt == st, "sequence and quality identifiers incongruent: %s != %s" % (qt, st)
            options.stdout.write( "@%s\n%s\n+\n%s\n" % (qt, ss, qs))

    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, noverflow=%i, nunderflow=%i\n" % \
                                  (iterator.mNInput, 
                                   iterator.mNOutput, 
                                   iterator.mNOverFlow, 
                                   iterator.mNUnderFlow ))

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperHmmer.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile, subprocess, optparse, shutil

from types import *

"""Wrapper for Hmmer
"""

# raise NotImplementedError("incomplete")

import Experiment
import TreeTools, MatrixTools

class Error(Exception):
    """Base class for exceptions in this module."""
    pass

    def __str__(self):
        return str(self.message)

class ParsingError(Error):
    """Exception raised for errors while parsing

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message, line):
        self.message = message + " at line " + line

class UsageError(Error):
    """Exception raised for errors while starting

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class HmmerSearchResult:
    def __init__(self):
        self.mNexus = None
        self.mOutfile = None

class Hmmer:

    def __init__( self ):
        
        self.mMapInput2Phylip = {}
        self.mMapPhylip2Input = {}
        
        self.mTempdir = None
        
        self.mInputTree = None
        self.mInputMatrix = None
        
        self.mOutputTree = "-"
        
        self.mOptions = ""

        self.mOutputStdout = None
        self.mOutputStderr = None

        self.mPruneTree = False

    def __del__(self):

        if self.mTempdir and self.mTempdir != "tmp":
            os.system( "rm -rf %s" % self.mTempdir )

    def setProgram( self, program ):
        self.mProgram = program

    def setOptions( self, options ):
        self.mOptions = options
        
    def setTree( self, tree ):
        self.mInputTree = tree

    def setMatrix( self, matrix):
        self.mInputMatrix = matrix

    def setPruneTree( self, flag = True ):
        self.mPruneTree = flag

    def updateMaps( self, taxa ):
        """update maps."""
        for taxon in taxa:
            if taxon not in self.mMapInput2Phylip:
                key = "t_%05i" % (len(self.mMapInput2Phylip) )
                self.mMapInput2Phylip[taxon] = key
                self.mMapPhylip2Input[key] = taxon


    def __reset(self):
        """clear internal state variables."""
        self.mMapInput2Phylip = {}
        self.mMapPhylip2Input = {}
        
        self.mTempdir = None
        

    def parseSearch( self, lines ):
        pass


if __name__ == "__main__":

    
    phylip = Phylip()

    parser = E.OptionParser( version = "%prog version: $Id: WrapperHmmer.py 2807 2009-10-22 13:42:10Z andreas $" )

    parser.add_option("-t", "--filename-input-tree", dest="filename_input_tree", type="string",
                      help="filename with tree information."  )
    parser.add_option("-T", "--filename-output-tree", dest="filename_output_tree", type="string",
                      help="output filename with tree information."  )
    parser.add_option("-p", "--program", dest="program", type="string",
                      help="program to use."  )
    parser.add_option("-o", "--options", dest="options", type="string",
                      help="input options."  )

    parser.set_defaults(
        filename_input_tree = None,
        filename_output_tree = None,
        program = None,
        options = "",
        )

    (options, args) = Experiment.Start( parser )

    if options.filename_input_tree != "-":
        phylip.setTree( open(options.filename_input_tree, "r").readlines() )
    elif options.filename_input_tree == "-":
        phylip.setTree( sys.stdin.readlines() )

    phylip.setOptions( options.options )
        
    phylip.setProgram( options.program )

    phylip.run()

    Experiment.Stop()
    



####
####
##
##
## Copyright (C) 2008 Andreas Heger All rights reserved
##
## Author: Andreas Heger <heger@ebi.ac.uk>
##
## $Id: psl2wiggle.py 2834 2009-11-24 16:11:23Z andreas $
##
##
####
####
'''
psl2wiggle.py - convert from psl to wiggle
==========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script converts from a :term:`psl` formatted
file to a :term:`wiggle` formatted file by stacking 
alignments on the target on top of each other.

This script uses the UCSC tools for bigwig/bigbed output.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

'''

import sys
import re
import string
import optparse
import time
import os
import itertools
import tempfile
import subprocess
import shutil

import CGAT.Experiment as E
import CGAT.Stats as Stats
import CGAT.Blat as Blat
import CGAT.IndexedFasta as IndexedFasta
import CGAT.IOTools as IOTools
import numpy

def main( argv = sys.argv ):

    parser = E.OptionParser( version = "%prog version: $Id: psl2wiggle.py 2834 2009-11-24 16:11:23Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome [default=%default]."  )

    parser.add_option("-b", "--output-filename", dest="output_filename", type="string",
                      help="filename for output [default=%default]" )

    parser.add_option("-o", "--output-format", dest="output_format", type="choice",
                      choices=("bedgraph", "wiggle", "bigbed", "bigwig"),
                      help="output format [default=%default]" )

    parser.set_defaults( genome_file = None,
                         typecode = numpy.int16,
                         output_filename = None,
                         output_format = "wiggle",
                         test = None )

    (options, args) = E.Start( parser, add_pipe_options = True )

    typecode = options.typecode

    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta(options.genome_file )
        counts = {}
        contig_sizes = fasta.getContigSizes( with_synonyms = False )
        E.info("allocating memory for %i contigs and %i bytes" % (len(contig_sizes), sum(contig_sizes.values()) * typecode().itemsize ))
        for contig,size in contig_sizes.items():
            E.debug( "allocating %s: %i bases" % (contig, size ) )
            counts[contig] = numpy.zeros( size, typecode )

        E.info("allocated memory for %i contigs" % len(fasta))

    else:
        fasta = None
        contig_sizes = {}

    if options.output_format in ("bigwig", "bigbed"):
        
        if not options.genome_file:
            raise ValueError("please supply genome file for bigwig/bigbed computation.")

        if not options.output_filename:
            raise ValueError("please output file for bigwig/bigbed computation.")

        if options.output_format == "bigwig":
            executable_name = "wigToBigWig"
        elif options.output_format == "bigbed":
            executable_name = "bedToBigBed"
        else:
            raise ValueError("unknown output format `%s`" % options.output_format)

        executable = IOTools.which( executable_name )

        if not executable:
            raise OSError( "could not find %s in path." % executable_name )

        tmpdir = tempfile.mkdtemp()
        E.debug( "temporary files are in %s" % tmpdir)

        tmpfile_wig = os.path.join( tmpdir, "wig" )
        tmpfile_sizes = os.path.join( tmpdir, "sizes" )

        # write contig sizes
        outfile_size = open( tmpfile_sizes, "w")
        for contig, size in contig_sizes.items():
            outfile_size.write("%s\t%s\n" % (contig, size) )
        outfile_size.close()    
        
        outfile = open( tmpfile_wig, "w" )

    else:
        outfile = options.stdout

    iterator = Blat.BlatIterator( sys.stdin )

    ninput, ncontigs, nskipped = 0, 0, 0

    E.info( "started counting" )

    while 1:

        if options.test and ninput >= options.test:
            break

        match = iterator.next()
        
        if match == None: break
        
        ninput += 1

        contig = match.mSbjctId

        for start, length in zip( match.mSbjctBlockStarts, match.mBlockSizes):
            counts[contig][start:start+length] += 1

    E.info( "finished counting" )

    if options.output_format in ("wig", "bigwig"):
        E.info( "starting wig output" )

        for contig, vals in counts.items():

            E.debug("output for %s" % contig )
            for val, iter in itertools.groupby( enumerate( vals ), lambda x: x[1] ):
                l = list(iter)
                start,end = l[0][0],l[-1][0]
                val = vals[start]
                if val > 0:
                    outfile.write("variableStep chrom=%s span=%i\n" % (contig, end-start+1))
                    outfile.write("%i\t%i\n" % (start, val) )

            ncontigs += 1
    elif options.output_format in ("bedgraph", "bigbed"):
        
        E.info( "starting bedgraph output" )

        for contig, vals in counts.items():
            E.debug("output for %s" % contig )
            for val, iter in itertools.groupby( enumerate( vals ), lambda x: x[1] ):
                l = list(iter)
                start,end = l[0][0],l[-1][0]
                val = vals[start]
                if val > 0:
                    outfile.write("%s\t%i\t%i\t%i\n" % (contig, start, end+1,val))
            
            ncontigs += 1

    E.info( "finished output" )

    if options.output_format in ("bigwig", "bigbed"):
        outfile.close()

        E.info( "starting bigwig conversion" )
        try:
            retcode = subprocess.call( " ".join( (executable,
                                                 tmpfile_wig,
                                                 tmpfile_sizes,
                                                 os.path.abspath( options.output_filename )), ),
                                       shell=True)
            if retcode < 0:
                warn( "wigToBigWig terminated with signal: %i" % -retcode)
                return -retcode
        except OSError, msg:
            warn( "Error while executing bigwig: %s" % e)
            return 1

        shutil.rmtree( tmpdir )
        
        E.info( "finished bigwig conversion" )

    E.info( "ninput=%i, ncontigs=%i, nskipped=%i\n" % (ninput, ncontigs, nskipped) )

    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv ) )
